import json
import sys
import time
from datetime import datetime

import click
from dateutil.parser import parse
from prettytable import PrettyTable, PLAIN_COLUMNS

import sdccli.usecases.policy.events as use_case
from sdccli.cli.policy.falco_list import falco_list
from sdccli.cli.policy.falco_macro import falco_macro
from sdccli.cli.policy.rule import rule
from sdccli.printer import print_list, print_item
from sdccli.time import duration_to_seconds

_item_keys_v1 = ["id", "name", "description", "falcoConfiguration", "actions", "scope", "notificationChannelIds"]
_list_keys_v1 = ["id", "name", "severity", "isBuiltin", "notificationChannelIds"]
_item_keys = ["id", "name", "description", "type", "severity", "enabled", "scope", "ruleNames", "actions",
              "notificationChannelIds"]
_list_keys = ["id", "name", "type", "severity", "enabled"]


@click.group(name='policy', short_help='Sysdig Secure policy operations')
def policy():
    pass


policy.add_command(rule)
policy.add_command(falco_list)
policy.add_command(falco_macro)


@policy.command(name='add', short_help="Add a policy")
@click.argument('name', nargs=1)
@click.argument('description', nargs=1)
@click.option('--rule', nargs=1, multiple=True,
              help="Rule name. It must be name instead of id, as the rules list view is by name, to account for multiple rules having the same name.")
@click.option('--action', nargs=1, multiple=True, help="It can be a stop, pause and/or capture action")
@click.option('--scope', nargs=1,
              help="Where the policy is being applied- Container, Host etc.. (example: 'container.image.repository = sysdig/agent')")
@click.option('--severity', nargs=1, default=0, type=click.INT,
              help="How severe is this policy when violated. Range from 0 to 7 included.")
@click.option('--notenabled', is_flag=True, help=" If the policy should not be considered")
@click.option('--notification', nargs=1, multiple=True, help="Notification channel id to subscribe to the policy.")
@click.option('--type', nargs=1, default="falco", type=click.STRING, help="Policy type, it can be one of: falco, list_matching, k8s_audit, aws_cloudvision")
@click.pass_obj
def add(cnf, name, description, rule, action, scope, severity, notenabled, notification, type):
    """
    NAME: the name of the new policy.

    DESCRIPTION: Description of policy
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2, use add-json instead")
        sys.exit(1)


    ok, res = cnf.sdsecure.add_policy(
        name, description,
        rule_names=rule,
        actions=action,
        scope=scope,
        severity=severity,
        enabled=not notenabled,
        notification_channels=notification,
        type=type)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='add-json', short_help="Add a policy from a json file")
@click.argument('policyfile', type=click.Path(exists=True), nargs=1)
@click.pass_obj
def add_json(cnf, policyfile):
    """
    POLICYFILE: A file with a json list of PolicyRule elements (while creating/updating a policy, new rule IDs will be created backend side)
    """
    try:
        with open(policyfile) as f:
            policy = f.read()
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (policyfile, str(error)))
        sys.exit(1)

    if cnf.sdsecure.policy_v2:
        ok, res = cnf.sdsecure.add_policy_json(policy)
    else:
        ok, res = cnf.sdsecure_v1.add_policy(policy)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if cnf.sdsecure.policy_v2:
            print_item(res, _item_keys)
        else:
            print_item(res["policy"], _item_keys_v1)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name="events", short_help="Get policy events")
@click.option('--duration', nargs=1, help="Duration to display the events from. The minimum is 10 minutes. "
                                          "(ex: 30M, 1H, 3D, 2W). Default: 3D.")
@click.option('--scope', nargs=1, help="this is a Sysdig Monitor-like filter (e.g 'kubernetes.cluster.name in "
                                       "(\"prod\", \"dev\")'). When provided, events are filtered by their scope, so "
                                       "only a subset will be returned (e.g. 'container.image.repo=\"ubuntu\"' will "
                                       "provide only events that have happened on an ubuntu container).")
@click.option('--severity', nargs=1, help="Filter by severity. Valid ones are: 'high', 'med', 'low', 'info'. "
                                          "Multiple ones can be specified if separated by commas. (e.g. 'med,low,info')")
@click.option('--type', nargs=1, help="Filter by event originator. Valid ones are: 'scanning', 'policy'. "
                                      "Multiple ones can be specified if separated by commas. (e.g. 'scanning,policy')")
@click.option('--search', nargs=1, help="Search by event title or label")
@click.option('--limit', nargs=1, type=click.INT, help="Limit the amount of events retrieved. Default: 50")
@click.argument('event-id', nargs=1, required=False)
@click.pass_obj
def events(cnf, event_id, duration, scope, severity, type, search, limit):
    try:
        if event_id:
            event = use_case.retrieve_policy_event_by_id(cnf.sdsecure, event_id)
            cnf.formatter.format(event, "policyEventV1")
            return


        events = use_case.retrieve_all_policy_events(cnf.sdsecure, duration, scope, severity, type, search, limit)
        cnf.formatter.format(events, "policyEventV1List")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@policy.command(name='events_old', short_help="Get policy events (Use only on old onprem installations)")
@click.argument('event-id', nargs=1, required=False)
@click.option('--duration', nargs=1, help="Duration to display the events from. (ex: 30M, 1H, 3D, 2W)")
@click.option('--start', nargs=1, help="Start of the time range.")
@click.option('--end', nargs=1, help="End of the time range.")
@click.option('--sampling', nargs=1, type=click.INT, default=60000000,
              help="sample all policy events using sampling interval.")
@click.option('--aggregate', nargs=1, multiple=True, default=['ruleType'],
              help="Specifies how to aggregate events (sampling does not need to be specified, because when it's "
                   "present it automatically means events will be aggregated). This field can either be a scope "
                   "metric or a policyEvent field but (currently) not a mix of the two. When policy events fields "
                   "are specified, only these can be used: severity, agentId, containerId, policyId, ruleType.")
@click.option('--scope', nargs=1,
              help="this is a SysdigMonitor-like filter (e.g 'container.image=ubuntu'). When provided, events are "
                   "filtered by their scope, so only a subset will be returned (e.g. 'container.image=ubuntu' will "
                   "provide only events that have happened on an ubuntu container).")
@click.option('--event-filter', nargs=1,
              help="this is a SysdigMonitor-like filter (e.g. policyEvent.policyId=3). When provided, events are "
                   "filtered by some of their properties. Currently the supported set of filters is "
                   "policyEvent.all(which can be used just with matches, policyEvent.policyId, policyEvent.id, "
                   "policyEvent.severity, policyEvent.ruleTye, policyEvent.ruleSubtype.")
@click.pass_obj
def events_old(cnf, event_id, duration, start, end, sampling, aggregate, scope, event_filter):
    """
    EVENT-ID: id of an event to display, if not provided a list of events will be displayed
    """
    kwargs = {
        "sampling": sampling,
        "aggregations": aggregate,
        "scope_filter": scope,
        "event_filter": event_filter
    }
    if duration:
        duration_sec = duration_to_seconds(duration)
        if duration_sec is None:
            return

        if event_id:
            ok, res = cnf.policy_events_client_old.get_policy_events_id_duration(event_id, duration_sec, **kwargs)
        else:
            ok, res = cnf.policy_events_client_old.get_policy_events_duration(duration_sec, **kwargs)

    else:
        if end:
            end_time = parse(end).timestamp()
        else:
            end_time = time.time()
        if start:
            start_time = parse(start).timestamp()
        else:
            start_time = end_time - (60 * 60)  # 1 hour

        if event_id:
            ok, res = cnf.policy_events_client_old.get_policy_events_id_range(event_id, start_time, end_time, **kwargs)
        else:
            ok, res = cnf.policy_events_client_old.get_policy_events_range(start_time, end_time, **kwargs)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if event_id:
            _print_event(res["data"]["policyEvent"])
        else:
            _print_events(res["data"]["policyEvents"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='list', short_help="List all policies")
@click.pass_obj
def list(cnf):
    if cnf.sdsecure.policy_v2:
        ok, res = cnf.sdsecure.list_policies()
    else:
        ok, res = cnf.sdsecure_v1.list_policies()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if cnf.sdsecure.policy_v2:
            print_list(res, _list_keys)
        else:
            print_list(res['policies'], _list_keys_v1)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='update', short_help="Update a policy")
@click.argument('id', nargs=1)
@click.option('--name', nargs=1, help="A short name for the policy")
@click.option('--description', nargs=1, help="Description of policy")
@click.option('--rule', nargs=1, multiple=True, help="Rule name. It will replace the rules defined in the policy")
@click.option('--action', nargs=1, multiple=True,
              help="It can be a stop, pause and/or capture action. It will replace the actions defined in the policy")
@click.option('--scope', nargs=1,
              help="Where the policy is being applied- Container, Host etc.. (example: 'container.image.repository = sysdig/agent')")
@click.option('--severity', nargs=1, type=click.INT,
              help="How severe is this policy when violated. Range from 0 to 7 included.")
@click.option('--notenabled', is_flag=True, help=" If the policy should not be considered")
@click.option('--notification', nargs=1, multiple=True, help="Notification channel id to subscribe to the policy.")
@click.pass_obj
def upd(cnf, id, name, description, rule, action, scope, severity, notenabled, notification):
    """
    ID: the id of the policy to update
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2, use add-json instead")
        sys.exit(1)

    if not rule:
        rule = None
    if not action:
        action = None
    if not notification:
        notification = None
    ok, res = cnf.sdsecure.update_policy(
        id,
        name=name,
        description=description,
        rule_names=rule,
        scope=scope,
        severity=severity,
        enabled=not notenabled,
        notification_channels=notification)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='update-json', short_help="Update a policy form a json file")
@click.argument('policyfile', type=click.Path(exists=True), nargs=1)
@click.pass_obj
def upd_json(cnf, policyfile):
    """
    POLICYFILE: A file with a json list of PolicyRule elements
    """
    try:
        with open(policyfile) as f:
            policy = f.read()
    except Exception as error:
        print("Error parsing rules file (%s): %s" % (policyfile, str(error)))
        sys.exit(1)

    if cnf.sdsecure.policy_v2:
        ok, res = cnf.sdsecure.update_policy_json(policy)
    else:
        ok, res = cnf.sdsecure_v1.update_policy(policy)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if cnf.sdsecure.policy_v2:
            print_item(res, _item_keys)
        else:
            print_item(res["policy"], _item_keys_v1)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='update-default', short_help="Create default policies using the current system falco rules")
@click.pass_obj
def update_default(cnf):
    """
    Create a set of default policies using the current system falco rules file as a reference. For every falco rule in the system
    falco rules file, one policy will be created. The policy will take the name and description from the name and description of
    the corresponding falco rule. If a policy already exists with the same name, no policy is added or modified. Existing
    policies will be unchanged.
    """
    if cnf.sdsecure.policy_v2:
        ok, res = cnf.sdsecure.create_default_policies()
    else:
        ok, res = cnf.sdsecure_v1.create_default_policies()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if cnf.sdsecure.policy_v2:
            print_list(res, _list_keys)
        else:
            print_list(res['policies'], _list_keys_v1)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='get', short_help="Get policy")
@click.argument('policy', nargs=1)
@click.pass_obj
def get(cnf, policy):
    """
    POLICY: Policy (id or name) to get
    """
    sdsecure = cnf.sdsecure
    if not cnf.sdsecure.policy_v2:
        sdsecure = cnf.sdsecure_v1

    try:
        id = int(policy)
        ok, res = sdsecure.get_policy_id(id)
    except ValueError:
        ok, res = sdsecure.get_policy(policy)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if cnf.sdsecure.policy_v2:
            print_item(res, _item_keys)
        else:
            print_item(res["policy"], _item_keys_v1)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@policy.command(name='del', short_help="Delete policies")
@click.argument('policies', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, policies):
    """
    POLICIES: Policy (id or name) to delete
    """
    sdsecure = cnf.sdsecure
    if not cnf.sdsecure.policy_v2:
        sdsecure = cnf.sdsecure_v1

    for policy in policies:
        try:
            id = int(policy)
            ok, res = sdsecure.delete_policy_id(id)
        except ValueError:
            ok, res = sdsecure.delete_policy_name(policy)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")


@policy.command(name='get-falco', short_help="Get a falco rules file")
@click.argument('outfile', type=click.File('w'), default=sys.stdout)
@click.option('--default', is_flag=True,
              help="Get the default falco rules file. (if not set the custom falco rules will be displayed)")
@click.pass_obj
def get_falco(cnf, outfile, default):
    """
    OUTFILE: File to write the rules into. If none is given the rules will be printed in stdout.
    """
    if default:
        ok, res = cnf.sdsecure.get_system_falco_rules()
    else:
        ok, res = cnf.sdsecure.get_user_falco_rules()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if not ok:
        print("Error: " + str(res))
        sys.exit(1)

    if default:
        outfile.write(res["systemRulesFile"]["content"])
    else:
        outfile.write(res)


@policy.command(name='set-falco', short_help="Set a falco rules file")
@click.argument('falcofile', type=click.File('r'), default=sys.stdin)
@click.option('--default', is_flag=True,
              help="Set the default falco rules file. (if not set the custom falco rules will be setted)")
@click.pass_obj
def set_falco(cnf, falcofile, default):
    """
    FALCOFILE: File with the falco rules. See the Falco wiki <https://github.com/draios/falco/wiki/Falco-Rules> for documentation on the falco rules format.
    """
    rules = falcofile.read()
    if default:
        ok, res = cnf.sdsecure.set_system_falco_rules(rules)
    else:
        ok, res = cnf.sdsecure.set_user_falco_rules(rules)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if not ok:
        print("Error: " + str(res))
        sys.exit(1)

    print("Success")


def _print_event(event):
    print_item(event, ["id", "name", "description"])
    if event["isAggregated"]:
        ts_start = _timestamp2datetime(event["minTimestamp"])
        ts_end = _timestamp2datetime(event["maxTimestamp"])
        print("{:25} {} - {}".format("time range:", ts_start, ts_end))
        rule_types = [t["element"] for t in event["ruleTypes"]]
        print("{:25} {}".format("Rule types:", rule_types))
        policy_ids = ["%s (%d)" % (p["element"], p["count"]) for p in event["policyIds"]]
        print("{:25} {}".format("policy ids:", policy_ids))
    else:
        timestamp = _timestamp2datetime(event["timestamp"])
        print("{:25} {}".format("timestamp:", timestamp))
        print("{:25} {:d}".format("policy id:", event["policyId"]))
    if event["isAggregated"]:
        print("\noutput:")
        for output in event["outputs"]:
            ocurrences = "(%d ocurrences) " % output["count"] if output["count"] > 1 else ""
            print("* {}{}".format(ocurrences, output["element"]))
    else:
        print("{:25} {}".format("output:", event["output"]))


def _print_events(events):
    t = PrettyTable(["id", "count", "severity", "name", "type"])
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for event in events:
        values = []
        values.append(event["id"])
        if event["isAggregated"]:
            values.append(event["aggregatedCount"])
            values.append(event["maxSeverity"])
        else:
            values.append(1)
            values.append(event["severity"])
        values.append(event["name"])
        if event["isAggregated"]:
            values.append(event["aggregatedMetrics"][1])
        else:
            values.append(event["ruleType"])
        t.add_row(values)
    print(t.get_string())


def _timestamp2datetime(timestamp):
    return datetime.utcfromtimestamp(timestamp / 10 ** 6)



from datetime import datetime

from sdccli.printer import print_item, print_list


def formats():
    return {
        "vulnerabilityDetails": _print_vuln_details,
        "vulnExceptionBundlesList": _print_vuln_bundle_list,
        "vulnExceptionBundlesAdded": _print_vuln_bundle_added,
        "vulnExceptionBundle": _print_vuln_budle,
        "vulnerabilityExceptionAdded": _print_vuln_added,
    }


def _print_vuln_details(vuln_detail):
    vuln_detail["description"] = vuln_detail["description"] or "No description found"
    print_item(vuln_detail, ["id", "severity", "description", "link"])
    if "nvd_data" in vuln_detail and vuln_detail["nvd_data"]:
        print("\nCVE Scores:")
        cvss_v2 = [{
            "id": f"{nvd_data['id']} CVSS v2",
            "severity": f"{nvd_data['cvss_v2']['severity']}",
            "access vector": f"{nvd_data['cvss_v2']['base_metrics']['access_vector']}",
            "base score": f"{nvd_data['cvss_v2']['base_metrics']['base_score']}",
            "exploitability score": f"{nvd_data['cvss_v2']['base_metrics']['exploitability_score']}",
            "impact score": f"{nvd_data['cvss_v2']['base_metrics']['impact_score']}",
        } for nvd_data in vuln_detail["nvd_data"] if "cvss_v2" in nvd_data and nvd_data["cvss_v2"]]

        print_list(cvss_v2, ["id", "severity", "access vector", "base score", "exploitability score", "impact score"])

    if "references" in vuln_detail and vuln_detail["references"]:
        print("\nReferences:")
        print_list(vuln_detail["references"], ["source", "url"])

    if "affected_packages" in vuln_detail and vuln_detail["affected_packages"]:
        print("\nAffected Packages:")
        print_list(vuln_detail["affected_packages"], ["name", "version"])


def _print_vuln_bundle_list(bundle_list):
    for bundle in bundle_list:
        bundle["item number"] = len(bundle["items"]) if "items" in bundle and bundle["items"] else 0

    print_list(bundle_list, ["id", "name", "comment", "item number"])


def _print_vuln_bundle_added(bundle):
    print_item(bundle, ["id", "name", "comment"])


def _print_vuln_budle(bundle):
    bundle["item number"] = len(bundle["items"]) if "items" in bundle and bundle["items"] else 0
    print_item(bundle, ["id", "name", "comment", "item number"])
    if "items" in bundle and bundle["items"]:
        print("\nItems:")
        items = [{
            "enabled": item["enabled"],
            "name": item["trigger_id"],
            "description": f"{item['description'][0:50]}..." if len(item["description"]) > 50 else (
                        item["description"] or "---"),
            "notes": item["notes"] if "notes" in item and item["notes"] else "---",
            "expiration date": datetime.fromtimestamp(item["expiration_date"]) if "expiration_date" in item and item[
                "expiration_date"] else "Never"
        } for item in bundle["items"]]
        print_list(items, ["enabled", "name", "description", "notes", "expiration date"])

def _print_vuln_added(vuln):
    item = {
        "enabled": vuln["enabled"],
        "name": vuln["trigger_id"],
        "description": f"{vuln['description'][0:100]}..." if len(vuln["description"]) > 100 else (
                vuln["description"] or "---"),
        "notes": vuln["notes"] if "notes" in vuln and vuln["notes"] else "---",
        "expiration date": datetime.fromtimestamp(vuln["expiration_date"]) if "expiration_date" in vuln and vuln[
            "expiration_date"] else "Never"
    }
    print_item(item, ["enabled", "name", "description", "notes", "expiration date"])


from prettytable import PrettyTable, PLAIN_COLUMNS

from sdccli.printer import print_item, print_list


def formats():
    return {
        "scanningImage": _print_image,
        "scanningImageAdded": _print_image_added,
        "scanningImageList": _print_images,
        "scanningQueryImage": _print_query,
        "scanningVulnImage": _print_vuln,
        "scanningEvaluationImage": _print_evaluation,
    }


def _print_image_added(images):
    for image in images:
        _print_image(image)


def _print_image(image):
    image["fulltag"] = [d["fulltag"] for d in image['image_detail']]
    if 'image_content' in image and image['image_content']:
        image_content = image['image_content']
        if 'metadata' in image_content and image_content['metadata']:
            image_content_metadata = image_content['metadata']
            image['dockerfile_mode'] = str(image_content_metadata['dockerfile_mode'])
            image['distro'] = str(image_content_metadata['distro'])
            image['distro_version'] = str(image_content_metadata['distro_version'])
            image['size'] = str(image_content_metadata['image_size'])
            image['arch'] = str(image_content_metadata['arch'])
            image['layer_count'] = str(image_content_metadata['layer_count'])

    keys = ["imageDigest", "imageId", "parentDigest", "analysis_status", "image_type", "fulltag",
            "dockerfile_mode", "distro", "distro_version", "size", "arch", "layer_count", "annotations"]
    print_item(image, keys)


def _print_query(res):
    res, query_type = res
    if not query_type:
        for t in res:
            print("%s: available" % (t,))
        return
    if query_type in ['manifest', 'dockerfile', 'docker_history']:
        try:
            print(res.get('metadata', "").decode('base64'))
        except AttributeError:
            print("No metadata %s" % (query_type,))
        return
    if "content" not in res:
        print("No content")
        return

    content = res["content"]
    keys = {
        'os': ['package', 'version', 'license'],
        'files': ['filename', 'size'],
        'npm': ['package', 'version', 'location'],
        'gem': ['package', 'version', 'location'],
        'python': ['package', 'version', 'location'],
        'java': ['package', 'specification-version', 'implementation-version', 'location']
    }
    if query_type in keys:
        print_list(content, keys[query_type])
    elif query_type in ['all', 'all+files']:
        for elem in content:
            content_type = elem["content_type"]
            print(f"# {content_type}:")
            _print_query((elem, content_type))
    else:
        print_list(content, content[0].keys())


def _print_vuln(res):
    res, query_type = res
    if "vulnerabilities" not in res:
        print("No vulnerabilities")
        return

    vulnerabilities = res["vulnerabilities"]
    if query_type in ['os', 'non-os', 'all']:
        keys = ['vuln', 'package', 'severity', 'fix', 'url']
        print_list(vulnerabilities, keys)
    else:
        print_list(vulnerabilities, vulnerabilities[0].keys())


def _print_images(images):
    header = ['Full Tag', 'Image ID', 'Analysis Status', 'Image Digest']
    t = PrettyTable(header)
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    add_rows = []
    for image_record in images:
        for image_detail in image_record['image_detail']:
            imageId = image_detail.get('imageId', "None")
            fulltag = image_detail.get('registry', "None") + "/" + image_detail.get('repo',
                                                                                    "None") + ":" + image_detail.get(
                'tag', "None")

            row = [fulltag, imageId, image_record['analysis_status'], image_record['imageDigest']]
            if row not in add_rows:
                add_rows.append(row)
    for row in add_rows:
        t.add_row(row)
    print(t.get_string(sortby='Full Tag'))


def _print_evaluation(res):
    scan_info, detail = res
    print_item(scan_info, ["image_tag", "status", "image_digest", "image_id", "total_stop",
                           "total_warn", "last_evaluation", "policy_id", "policy_name"])

    if not detail:
        return

    if scan_info["warn_results"]:
        print("Warn results:")
        for warn_result in scan_info["warn_results"]:
            print(f"  - {warn_result}")

    if scan_info["stop_results"]:
        print("Stop results:")
        for stop_result in scan_info["stop_results"]:
            print(f"  x {stop_result}")



from sdccli.printer import print_list, print_item


def format_alert_list(data):
    print_list(data, ["id", "name", "type", "enabled", "severityLabel"])


def format_alert(data):
    print_item(data,
               ["id", "name", "description", "type", "enabled", "severityLabel", "segmentBy", "segmentCondition",
                "condition", "timespan"])


def formats():
    return {
        "alertList": format_alert_list,
        "alert": format_alert,
    }



import click
import json
import sys
from sdccli.printer import print_list, print_item

_list_printable_column_name = ["ID", "status", "image", "network", "processes", "filesystem", "syscalls"]

column_name_to_json_map = {
    "ID"            : "profileId",
    "status"        : "status",
    "image"         : "imageName",
    "network"       : "networkProposal",
    "processes"     : "processesProposal",
    "filesystem"    : "fileSystemProposal",
    "syscalls"      : "syscallProposal"
}

supported_rule_types = ["NETWORK", "SYSCALL", "FILESYSTEM", "PROCESS"]


@click.group(name='profile', short_help='Sysdig Secure image profile operations')
def profile():
    pass


@profile.command(name='list', short_help="List all profiles")
@click.pass_obj
def list_profiles(cnf):
    ok, res = cnf.sdsecure.list_image_profiles()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    # Filter out some fields that will not be printed
    filtered_profiles = []
    for profile in res['profiles']:
        filtered_profiles.append(__map_profile_to_printable_format(profile))

    if ok:
        print_list(filtered_profiles, _list_printable_column_name)
    else:
        print("Error: " + str(res))
        sys.exit(1)


def __map_profile_to_printable_format(raw_profile, fullID=False):
    '''
    **Description**
        Return printable data structure of profile list

    **Arguments**
        - raw_profile: dictionary representing the "profile" field in the json
        - fullID: print or not the first 12 characters of the profile ID

    **Success Return Value**
        Return a dictionary with renamed keys (and lesser number of fields)
    '''

    filt_profile = {}

    for column in _list_printable_column_name:
        if column in ["ID"]:
            if fullID:
                filt_profile[column] = raw_profile[column_name_to_json_map[column]]
            else:
                filt_profile[column] = raw_profile[column_name_to_json_map[column]][0:12]
        elif column in ["status", "image"]:
            filt_profile[column] = raw_profile[column_name_to_json_map[column]]
        else:
            filt_profile[column] = raw_profile[column_name_to_json_map[column]]["score"]

    return filt_profile


@profile.command(name='get', short_help="Get profile by ID")
@click.argument('id', nargs=1)
@click.argument('rule_type', nargs=1)
@click.pass_obj
def get(cnf, id, rule_type):
    """
    PROFILE: PROFILE_ID RULE_TYPE

    Supported RULE_TYPE=[NETWORK | SYSCALL | FILESYSTEM | PROCESS]
    """

    if rule_type not in supported_rule_types:
        print("Error: No such rule type: {}\nRule type list: NETWORK, SYSCALL, FILESYSTEM, PROCESS".format(rule_type))
        sys.exit(1)

    ok, res = cnf.sdsecure.get_image_profile(id)
    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if not ok:
        if isinstance(res, list):
            # Collision detected
            print("Collision detected: " + str(len(res)))
            __print_collision_table(res)
        else:
            # Generic error. For example: ID not found.
            print("Error: " + str(res))

        sys.exit(1)

    rule_found = False

    # If the res is None, it means that there are no information
    # associated to that profile. Print a general message and return.
    if res is None:
        print("Profiles are still in learning state.")
        return

    # Given the provided rule type, return the relevant information
    # to the user. At this moment, there is no possibility to retrieve all
    # the information at once. This is done to be consistent with the usability
    # of the cli.
    # Moreover, please note that the printed subfields depend on the rule type
    for rule in res['proposedRules']:
        ruleDict = {}

        if rule['details']['ruleType'] != rule_type:
            continue

        if rule['details']['ruleType'] == 'NETWORK':
            ruleDict['name'] = rule['name'].split('-')[0].strip()
            ruleDict["size"] = len(rule['details']['tcpListenPorts']['items']) + len(rule['details']['udpListenPorts']['items'])
            ruleDict["tcpListenPorts"] = rule['details']['tcpListenPorts']['items']
            ruleDict["udpListenPorts"] = rule['details']['udpListenPorts']['items']

            print("\n{}".format(ruleDict['name']))
            print_item(ruleDict, ['size', 'tcpListenPorts', 'udpListenPorts'])

            rule_found = True

        elif rule['details']['ruleType'] == 'FILESYSTEM':
            ruleDict['name'] = rule['name'].split('-')[0].strip()
            ruleDict["size"] = len(rule['details']['readWritePaths']['items']) + len(rule['details']['readPaths']['items'])
            ruleDict["readWritePaths"] = rule['details']['readWritePaths']['items']
            ruleDict["readPaths"] = rule['details']['readPaths']['items']

            print("\n{}".format(ruleDict['name']))
            print_item(ruleDict, ['size', 'readWritePaths', 'readPaths'])

            rule_found = True

        elif rule['details']['ruleType'] == 'PROCESS':
            ruleDict['name'] = rule['name'].split('-')[0].strip()
            ruleDict["size"] = len(rule['details']['processes']['items'])
            ruleDict["processes"] = rule['details']['processes']['items']

            print("\n{}".format(ruleDict['name']))
            print_item(ruleDict, ['size', 'processes'])

            rule_found = True

        elif rule['details']['ruleType'] == 'SYSCALL':
            ruleDict['name'] = rule['name'].split('-')[0].strip()
            ruleDict['size'] = len(rule['details']['syscalls']['items'])
            ruleDict['syscalls'] = rule['details']['syscalls']['items']

            print("\n{}".format(ruleDict['name']))
            print_item(ruleDict, ['size', 'syscalls'])

            rule_found = True

    # If the rule_type is not found, print the available  rule types
    if not rule_found:
        print("No data for rule {}".format(
            rule_type
        ))


def __print_collision_table(collisions):
    '''
    **Description**
        Helper function to print the collision table. The user is able to see which are the profiles
        with the common ID prefix (remember that the user can retrieve a profile by the prefix substring of the
        profile ID).

    **Arguments**
        - List of dictionaries, where each dictionary represents the "profiles" field in the retrieved json

    **Success Return Value**
        None. Print the collision table
    '''

    columns = ["imageName", "profileId"]

    table = [{columns[0]: profile[columns[0]], columns[1]: profile[columns[1]]} for profile in collisions]

    print_list(table, columns)



from datetime import datetime, timedelta

from dateutil.parser import parse


def date_range_in_sec(start, end, duration=None):
    if not duration:
        duration = '1D'

    duration_secs = duration_to_seconds(duration)
    to_sec = parse(end).timestamp() if end else 0
    from_sec = parse(start).timestamp() if start else 0

    if not from_sec and not to_sec:
        to_sec = (datetime.utcnow() - datetime.utcfromtimestamp(0)).total_seconds()

    if from_sec > to_sec:
        raise IndexError("The start date is after the end date: {} <-> {}".format(parse(start), parse(end)))

    from_sec = from_sec if from_sec else to_sec - duration_secs
    to_sec = to_sec if to_sec else from_sec + duration_secs
    return from_sec, to_sec


def duration_to_timedelta(duration):
    return timedelta(seconds=(duration_to_seconds(duration)))


def duration_to_seconds(duration):
    format_error = "Error: duration needs to be in the form of <num><units> where <units> can be: S, M, H, D or W. (ex: 30M, 1H, 3D, 2W)"
    unit_sec = {
        "s": 1,
        "m": 60,
        "h": 60 * 60,
        "d": 60 * 60 * 24,
        "w": 60 * 60 * 24 * 7
    }
    if duration[-1].lower() not in unit_sec.keys():
        print(format_error)
        return None

    try:
        return int(duration[:-1]) * unit_sec[duration[-1].lower()]
    except ValueError:
        print(format_error)
    return None


def format_timestamp(timestamp):
    return str(datetime.utcfromtimestamp(timestamp / 10 ** 3))



from sdccli.printer import print_item, print_list

_item_keys = ["alertId", "name", "enabled", "description", "scope", "triggers",
              "notificationChannelIds", "createdAt", "updatedAt"]
_list_keys = ["alertId", "name", "enabled"]


def formats():
    return {
        "scanningAlertList": format_scanning_alert_list,
        "scanningAlert": format_scanning_alert,
    }


def format_scanning_alert(alert):
    print_item(alert, _item_keys)


def format_scanning_alert_list(alert_list):
    print_list(alert_list, _list_keys)



from abc import ABC, abstractmethod

import yaml

import sdccli.drop as drop
import sdccli.restore as restore


def _same_dict(d1, d2, skip_keys):
    if (d1.keys() - skip_keys) != (d2.keys() - skip_keys):
        return False

    for k in d1.keys():
        if k in skip_keys:
            continue
        if d1[k] != d2[k]:
            return False

    return True


class RemoveUseCase(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def execute(self):
        pass


class RestoreUseCase(ABC):
    def __init__(self, repository, formatter=yaml.safe_load):
        self._repository = repository
        self._formatter = formatter

    @abstractmethod
    def execute(self):
        pass

    def _load_data_from_repo(self, kind):
        file_data = self._formatter(self._repository.read())
        if file_data is None:
            return []
        file_data = [data["spec"] for data in file_data if data["kind"] == kind]
        return file_data


class RestoreMonitorUseCase(RestoreUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        self._monitor = monitor
        super().__init__(repository, formatter)


class RestoreSecureUseCase(RestoreUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        self._secure = secure
        super().__init__(repository, formatter)


class RestoreUsersUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("users")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        ok, data = self._monitor.get_users()
        if not ok:
            return False, data

        existing_user_list = [user["username"] for user in data]
        errors = []
        users_to_invite = [user for user in file_data if not user["username"] in existing_user_list]
        total = len(users_to_invite)
        for i, user in enumerate(users_to_invite):
            print("[{}/{}] Creating user {}... ".format(i + 1, total, user['username']))
            ok, res = self._monitor.create_user_invite(user["username"], user.get("firstName", ""),
                                                       user.get("lastName", ""),
                                                       user["systemRole"])
            if not ok:
                errors.append("couldn't recreate user {} : {}\n".format(user["username"], res))

        if len(errors) != 0:
            return False, errors
        return True, None


class RestoreNotificationChannelsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("notification_channels")
        file_data = [item for sublist in file_data for item in sublist['notificationChannels']]  # Flatten lists

        _, res = self._monitor.list_notification_channels()
        existing_channels = {n['name']: n for n in res['notificationChannels']}

        return restore.notification_channels(file_data, self._monitor, existing_channels)


class RemoveAndRestoreNotificationChannelsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("notification_channels")
        file_data = [item for sublist in file_data for item in sublist['notificationChannels']]  # Flatten lists

        drop.notification_channels(self._monitor)
        return restore.notification_channels(file_data, self._monitor, existing_channels={})


class RestoreOwnedDashboardsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("dashboards")
        file_data = [item for sublist in file_data for item in sublist['dashboards']]  # Flatten lists

        ok, res = self._monitor.get_dashboards()
        if not ok:
            return False, 'Error restoring dashboards: {}'.format(res)

        existing_dashboards = {d['name']: d for d in res['dashboards']}

        return restore.dashboards(file_data, self._monitor, existing_dashboards, restore_unowned=False)


class RestoreAllDashboardsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("dashboards")
        file_data = [item for sublist in file_data for item in sublist['dashboards']]  # Flatten lists

        ok, res = self._monitor.get_dashboards()
        if not ok:
            return False, 'Error restoring dashboards: {}'.format(res)

        existing_dashboards = {d['name']: d for d in res['dashboards']}

        return restore.dashboards(file_data, self._monitor, existing_dashboards, restore_unowned=True)


class RemoveOwnedAndRestoreOwnedDashboardsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("dashboards")
        file_data = [item for sublist in file_data for item in sublist['dashboards']]  # Flatten lists

        ok, res = drop.dashboards(self._monitor, remove_unowned=False)
        if not ok:
            return ok, res
        return restore.dashboards(file_data, self._monitor, existing_dashboards={}, restore_unowned=False)


class RemoveAllAndRestoreAllDashboardsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("dashboards")
        file_data = [item for sublist in file_data for item in sublist['dashboards']]  # Flatten lists

        ok, res = drop.dashboards(self._monitor, remove_unowned=True)
        if not ok:
            return ok, res
        return restore.dashboards(file_data, self._monitor, existing_dashboards={}, restore_unowned=True)


class RestoreAlertsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("alerts")
        file_data = [item for sublist in file_data for item in sublist['alerts']]  # Flatten lists

        return restore.alerts(file_data, self._monitor)


class RemoveAndRestoreAlertsUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("alerts")
        file_data = [item for sublist in file_data for item in sublist['alerts']]  # Flatten lists

        ok, res = drop.alerts(self._monitor)
        if not ok:
            return False, res

        return restore.alerts(file_data, self._monitor)


class RestoreTeamsMonitorUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("monitor_teams")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        return restore.teams(file_data, self._monitor)


class RemoveAndRestoreTeamsMonitorUseCase(RestoreMonitorUseCase):
    def __init__(self, monitor, repository, formatter=yaml.safe_load):
        super().__init__(monitor, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("monitor_teams")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        ok, res = drop.teams(self._monitor)
        if not ok:
            return False, res

        return restore.teams(file_data, self._monitor)


class RestoreTeamsSecureUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("secure_teams")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        return restore.teams(file_data, self._secure)


class RemoveAndRestoreTeamsSecureUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("secure_teams")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        ok, res = drop.teams(self._secure)
        if not ok:
            return False, res

        return restore.teams(file_data, self._secure)


class RestoreFalcoMacrosUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("falco_macros")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        return restore.falco_macros(file_data, self._secure)


class RemoveAndRestoreFalcoMacrosUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("falco_macros")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        ok, res = drop.falco_macros(self._secure)
        if not ok:
            return False, res
        return restore.falco_macros(file_data, self._secure)


class RestoreFalcoListsUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("falco_lists")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        return restore.falco_lists(file_data, self._secure)


class RemoveAndRestoreFalcoListsUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("falco_lists")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        ok, res = drop.falco_lists(self._secure)
        if not ok:
            return False, res
        return restore.falco_lists(file_data, self._secure)


class RestoreFalcoRulesUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("falco_rules")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        return restore.rules(file_data, self._secure)


class RemoveAndRestoreFalcoRulesUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("falco_rules")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        ok, res = drop.rules(self._secure)
        if not ok:
            return False, res
        return restore.rules(file_data, self._secure)


class RestorePoliciesUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("policies")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        return restore.policies(file_data, self._secure)


class RemoveAndRestorePoliciesUseCase(RestoreSecureUseCase):
    def __init__(self, secure, repository, formatter=yaml.safe_load):
        super().__init__(secure, repository, formatter)

    def execute(self):
        file_data = self._load_data_from_repo("policies")
        file_data = [item for sublist in file_data for item in sublist]  # Flatten lists

        ok, res = drop.policies(self._secure)
        if not ok:
            return False, res

        return restore.policies(file_data, self._secure)


class RemoveSecureUseCase(RemoveUseCase):
    def __init__(self, secure):
        self._secure = secure


class RemovePoliciesUseCase(RemoveSecureUseCase):
    def __init__(self, secure):
        super().__init__(secure)

    def execute(self):
        return drop.policies(self._secure)


class RemoveFalcoRulesUseCase(RemoveSecureUseCase):
    def __init__(self, secure):
        super().__init__(secure)

    def execute(self):
        return drop.rules(self._secure)


class RemoveFalcoListsUseCase(RemoveSecureUseCase):
    def __init__(self, secure):
        super().__init__(secure)

    def execute(self):
        return drop.falco_lists(self._secure)


class RemoveFalcoMacrosUseCase(RemoveSecureUseCase):
    def __init__(self, secure):
        super().__init__(secure)

    def execute(self):
        return drop.falco_macros(self._secure)



import os
import os.path

import yaml
from sdcclient import SdMonitorClient, SdMonitorClient as SdMonitorClientV2, SdSecureClient, SdScanningClient, \
    SdSecureClientV1
from sdcclient.monitor import EventsClientV1
from sdcclient.secure import PolicyEventsClientOld

from sdccli.cli.formatter.json_formatter import JsonFormatter
from sdccli.cli.formatter.text_formatter import NormalFormatter


class Config(object):
    __default_config_paths = [
        os.path.expanduser("~") + "/.config/sdc-cli/config.yml",
        "/etc/sdc-cli/config.yml",
        "/config.yml"
    ]
    __default_monitor_url = 'https://app.sysdigcloud.com'
    __default_secure_url = 'https://secure.sysdig.com'

    def __init__(self, json=False):
        self.json = json
        self.formatter = JsonFormatter() if json else NormalFormatter()
        self.monitor = {}
        self.secure = {}

    def load_from_text(self, text, env=None):
        try:
            config = yaml.load(text, Loader=yaml.SafeLoader)
        except yaml.YAMLError as exc:
            raise Exception("error in configuration file:", exc)

        if "envs" not in config:
            raise Exception("config file does not have a envs parent")

        if env not in config["envs"]:
            raise Exception("environment provided '{}' not found in the configuration file, "
                            "envs found: {}".format(env, list(config['envs'].keys())))

        self.__config_with_defaults(config["envs"][env])

    def load(self, path=None, env=None):
        env = env if env is not None else os.getenv("SDC_ENV", "main")

        if path is None or path == "":
            found = False
            for file_path in self.__default_config_paths:
                if os.path.isfile(file_path):
                    path = file_path
                    found = True
            if not found:
                self.__config_with_defaults()
                return
        else:
            if not os.path.isfile(path):
                raise EnvironmentError("couldn't find the provided config file: {}".format(path))

        with open(path, "r") as f:
            self.load_from_text(f.read(), env)

    def __config_with_defaults(self, config=None):
        if config is None:
            config = {}

        monitor_token = os.environ.get('SDC_TOKEN')
        if os.environ.get('SDC_MONITOR_TOKEN'):
            monitor_token = os.environ.get('SDC_MONITOR_TOKEN')
        secure_token = os.environ.get('SDC_TOKEN')
        if os.environ.get('SDC_SECURE_TOKEN'):
            secure_token = os.environ.get('SDC_SECURE_TOKEN')

        monitor_url = self.__default_monitor_url
        if os.environ.get('SDC_MONITOR_URL'):
            monitor_url = os.environ.get('SDC_MONITOR_URL')
        secure_url = self.__default_secure_url
        if os.environ.get('SDC_SECURE_URL'):
            secure_url = os.environ.get('SDC_SECURE_URL')

        if 'monitor' in config:
            self.monitor = config["monitor"]
        if monitor_token:
            self.monitor['token'] = monitor_token
        if 'url' not in self.monitor:
            self.monitor['url'] = monitor_url
        self.monitor['ssl_verify'] = not self.monitor.get('disable_ssl_verification', False)

        if 'secure' in config:
            self.secure = config["secure"]
        if secure_token:
            self.secure['token'] = secure_token
        if 'url' not in self.secure:
            self.secure['url'] = secure_url
        self.secure['ssl_verify'] = not self.secure.get('disable_ssl_verification', False)

        self.monitor['url'] = self.monitor['url'].rstrip('/')
        self.secure['url'] = self.secure['url'].rstrip('/')

        if not (self.monitor['ssl_verify'] and self.secure['ssl_verify']):
            # suppress the InsecureRequestWarning when non tls connections are created
            import urllib3
            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

    @property
    def events_client_v1(self):
        if "token" not in self.monitor:
            raise PermissionError("attempt to execute a Monitor operation without providing a Monitor token")
        return EventsClientV1(
            token=self.monitor["token"],
            sdc_url=self.monitor["url"],
            ssl_verify=self.monitor["ssl_verify"],
            custom_headers=self.monitor.get("extra_headers", {})
        )

    @property
    def sdmonitor(self):
        if "token" not in self.monitor:
            raise PermissionError("attempt to execute a Monitor operation without providing a Monitor token")
        return SdMonitorClient(
            token=self.monitor["token"],
            sdc_url=self.monitor["url"],
            ssl_verify=self.monitor["ssl_verify"],
            custom_headers=self.monitor.get("extra_headers", {})
        )

    @property
    def sdmonitor_v2(self):
        if "token" not in self.monitor:
            raise PermissionError("attempt to execute a Monitor operation without providing a Monitor token")
        return SdMonitorClientV2(
            token=self.monitor["token"],
            sdc_url=self.monitor["url"],
            ssl_verify=self.monitor["ssl_verify"],
            custom_headers=self.monitor.get("extra_headers", {})
        )

    @property
    def policy_events_client_old(self):
        if "token" not in self.monitor:
            raise PermissionError("attempt to execute a Monitor operation without providing a Monitor token")
        return PolicyEventsClientOld(
            token=self.monitor["token"],
            sdc_url=self.monitor["url"],
            ssl_verify=self.monitor["ssl_verify"],
            custom_headers=self.monitor.get("extra_headers", {})
        )

    @property
    def sdsecure(self):
        if "token" not in self.secure:
            raise PermissionError("attempt to execute a Secure operation without providing a Secure token")
        return SdSecureClient(
            token=self.secure["token"],
            sdc_url=self.secure["url"],
            ssl_verify=self.secure["ssl_verify"],
            custom_headers=self.secure.get("extra_headers", {})
        )

    @property
    def sdsecure_v1(self):
        if "token" not in self.secure:
            raise PermissionError("attempt to execute a Secure operation without providing a Secure token")
        return SdSecureClientV1(
            token=self.secure["token"],
            sdc_url=self.secure["url"],
            ssl_verify=self.secure["ssl_verify"],
            custom_headers=self.secure.get("extra_headers", {})
        )

    @property
    def sdscanning(self):
        if "token" not in self.secure:
            raise PermissionError("attempt to execute a Secure operation without providing a Secure token")
        return SdScanningClient(
            token=self.secure["token"],
            sdc_url=self.secure["url"],
            ssl_verify=self.secure["ssl_verify"],
            custom_headers=self.secure.get("extra_headers", {})
        )



import csv
import json
import sys
from datetime import timezone
from multiprocessing.pool import ThreadPool

import click
import dateutil

COLUMN_MAP = {
    "image": None,
    "type": None,
    "imageDigest": None,
    "feed": "feed",
    "feed_group": "feed_group",
    "fix": "fix",
    "package": "package",
    "package_cpe": "package_cpe",
    "package_name": "package_name",
    "package_path": "package_path",
    "package_type": "package_type",
    "package_version": "package_version",
    "severity": "severity",
    "url": "url",
    "vuln": "vuln",
    "whitelisted": None,
    "cvss_v2_base_score": ("nvd_data", 0, "cvss_v2", "base_score"),
    "cvss_v2_exploitability_score": ("nvd_data", 0, "cvss_v2", "exploitability_score"),
    "cvss_v2_impact_score": ("nvd_data", 0, "cvss_v2", "impact_score"),
    "cvss_v3_base_score": ("nvd_data", 0, "cvss_v3", "base_score"),
    "cvss_v3_exploitability_score": ("nvd_data", 0, "cvss_v3", "exploitability_score"),
    "cvss_v3_impact_score": ("nvd_data", 0, "cvss_v3", "impact_score"),
}


# TODO: Merging or sorting?
# TODO: Multiple digests for same tag! Or multiple tags for each image. Causing duplicates. Probably we could save time recovering by digest, and in fact I am not sure if we are recovering issues correctly.

@click.command(name='cve-report', short_help="(DEPRECATED Use 'sdc-cli scanning vulnerability report') Get a CVE report")
@click.option("--runtime", help="Generate CVE report for runtime images only", type=bool)
@click.option("--scope",
              help="Prefix to filter the images in the report. i.e. 'gcr.io/sysdig/sysdig-agent:latest' or 'docker.io/'")
@click.option("--date",
              help="Specify a date for the report. Only images created after this date will be included in the report. Use format YYYY-MM-DD or \"YYYY-MM-DD HH:MM\"")
@click.option("--columns", help="List of comma-separated colums. Available columns: %s" % ", ".join(COLUMN_MAP),
              default="image,imageDigest,type,vuln,whitelisted,package_path,fix,cvss_v3_base_score,cvss_v2_base_score,severity,package,url",
              show_default=True)
@click.argument('report_path', nargs=1, required=False)
@click.pass_obj
def cve_report(cnf, runtime, scope, date, columns, report_path):
    """
    (DEPRECATED Use 'sdc-cli scanning vulnerability report')
    """
    print("WARNING. This option has been deprecated, please use 'sdc-cli scanning vulnerability report'")
    if not cnf.json and not report_path:
        print("Missing REPORT_PATH argument")
        sys.exit(1)

    parsed_date = None
    try:
        if date:
            parsed_date = dateutil.parser.parse(date).replace(tzinfo=timezone.utc)
    except dateutil.parser.ParserError as e:
        print("Error parsing date:\n%s" % e)
        sys.exit(1)

    column_list = columns.split(",")
    for col in column_list:
        if col not in COLUMN_MAP:
            print("Invalid column: %s" % col)
            sys.exit(1)

    if runtime:
        if parsed_date:
            print("Date filtering not supported for --runtime")
            sys.exit(1)
        images = get_runtime_images(cnf)
        if len(images) == 0:
            print("Warning: no runtime images detected, the report will be empty")
    else:
        images = get_all_images(cnf)

    filtered_images = filter_images(images, scope, parsed_date)

    cve_whitelist = get_whitelisted_cves(cnf)

    cves = []

    for idx, (ok, image_cves) in enumerate(
            ThreadPool().map(lambda img:
                             add_cves_for(cnf, img, cve_whitelist, column_list), filtered_images)
    ):
        if not ok:
            return print_error(cnf, image_cves, exit_app=False)

        cves.extend(image_cves)

    if cnf.json:
        print(json.dumps(cves, indent=4))
    else:
        export_cvs_report(report_path, cves, column_list)
        print("Report %s saved" % report_path)


def get_all_images(cnf):
    images = []
    ok, res = cnf.sdscanning.list_images()
    if not ok:
        return print_error(cnf, res)
    for image in res:
        if image["analysis_status"] != "analyzed":
            continue
        for image in image["image_detail"]:
            images.append(image)
    return images


def get_runtime_images(cnf):
    images = []
    ok, res = cnf.sdscanning.list_runtime()
    if not ok:
        return print_error(cnf, res)

    for image in res["images"]:
        if image["analysisStatus"] != "analyzed":
            continue
        images.append({
            "imageDigest": image["digest"],
            "fulldigest": image["repo"] + "@" + image["digest"],
            "fulltag": image["repo"] + ":" + image["tag"]
        })
    return images


def get_whitelisted_cves(cnf):
    whitelisted_cves = set([])
    ok, bundles = cnf.sdscanning.list_vulnerability_exception_bundles()
    if not ok:
        return print_error(cnf, bundles)

    for bundle in bundles:
        ok, res = cnf.sdscanning.get_vulnerability_exception_bundle(bundle=bundle["id"])
        if not ok:
            return print_error(cnf, res)
        for item in res["items"]:
            whitelisted_cves.add(item["trigger_id"])

    return whitelisted_cves


def export_cvs_report(report_path, cves, column_list):
    with open(report_path, 'w', newline='') as csvfile:
        writer = csv.writer(csvfile, dialect='excel')
        writer.writerow(column_list)
        for cve in cves:
            row = []
            for col in column_list:
                row.append(cve[col])
            writer.writerow(row)


def filter_images(images, scope, date):
    filtered_images = []
    for image in images:

        if scope and not image["fulltag"].startswith(scope):
            continue

        if date and "created_at" in image and dateutil.parser.parse(image["created_at"]) < date:
            continue

        filtered_images.append(image)

    return filtered_images


def print_error(cnf, res, exit_app=True):
    if cnf.json:
        print(json.dumps(res, indent=4))
        sys.exit(1)

    else:
        print("Error: " + str(res))
        if exit_app:
            sys.exit(1)


def add_cves_for(cnf, image_detail, cve_whitelist, column_list):
    if not cnf.json:
        print(f"Recovering vulnerability report for: {image_detail['fulltag']} ({image_detail['imageDigest']}) ")
    cves = []
    for vuln_type in ["os", "non-os"]:
        # Recovering by digest, as it makes more sense
        ok, res = cnf.sdscanning.query_image_vuln(image_detail["fulldigest"], vuln_type, vendor_only=True)
        if not ok:
            return False, res
        cves.extend(flatten_vuln_list(cnf, image_detail, vuln_type, cve_whitelist, column_list, res["vulnerabilities"]))

    return True, cves


def flatten_vuln_list(cnf, image_detail, vuln_type, cve_whitelist, column_list, vulnerabilities):
    flat_vulnerabilities = []
    for vuln in vulnerabilities:
        flat_vuln = {}
        for col in column_list:
            target = COLUMN_MAP[col]

            if col == "type":
                flat_vuln[col] = vuln_type
            elif col == "image":
                flat_vuln[col] = image_detail["fulltag"]
            elif col == "imageDigest":
                flat_vuln[col] = image_detail["imageDigest"]
            elif col == "whitelisted":
                flat_vuln[col] = vuln["vuln"] in cve_whitelist
            elif isinstance(target, str):
                flat_vuln[col] = vuln[target]
            elif isinstance(target, tuple):
                try:
                    value = vuln
                    for k in target:
                        value = value[k]
                except:
                    value = None
                flat_vuln[col] = value
        flat_vulnerabilities.append(flat_vuln)
    return flat_vulnerabilities




def annotation_arguments_to_map(annotation):
    annotations = {}
    if annotation:
        for a in annotation:
            try:
                (k, v) = a.split('=', 1)
                if not k:
                    raise KeyError(f"found null key for annotation {a}")

                if not v:
                    raise AttributeError(f"found null value for annotation {a}")

                annotations[k] = v
            except ValueError:
                raise ValueError(f"annotation format error - annotations must be of the form "
                                 f"(--annotation key=value), found: {a}")
    return annotations



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ['registry', 'registry_user', 'registry_type', 'registry_verify', 'created_at', 'last_updated']
_list_keys = ["registry", "registry_type", "registry_user"]


@click.group(name='registry', short_help='Registry operations')
def registry():
    pass


@registry.command(name='add', short_help="Add a registry")
@click.argument('registry', nargs=1, required=True)
@click.argument('registry_user', nargs=1, required=True)
@click.argument('registry_pass', nargs=1, required=True)
@click.option('--insecure', is_flag=True, default=False, help="Allow connection to registry without SSL cert checks (ex: if registry uses a self-signed SSL certificate)")
@click.option('--registry-type', help="Specify the registry type (default='docker_v2')")
@click.option('--skip-validate', is_flag=True, help="Do not attempt to validate registry/creds on registry add")
@click.pass_obj
def add(cnf, registry, registry_user, registry_pass, insecure, registry_type, skip_validate):
    """
    REGISTRY: Full hostname/port of registry. Eg. myrepo.example.com:5000

    REGISTRY_USER: Username

    REGISTRY_PASS: Password
    """
    ok, res = cnf.sdscanning.add_registry(
        registry, registry_user, registry_pass,
        insecure=insecure,
        registry_type=registry_type,
        validate=not skip_validate)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res[0], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@registry.command(name='update', short_help="Update an existing registry")
@click.argument('registry', nargs=1, required=True)
@click.argument('registry_user', nargs=1, required=True)
@click.argument('registry_pass', nargs=1, required=True)
@click.option('--insecure', is_flag=True, default=False, help="Allow connection to registry without SSL cert checks (ex: if registry uses a self-signed SSL certificate)")
@click.option('--registry-type', default='docker_v2', help="Specify the registry type (default='docker_v2')")
@click.option('--skip-validate', is_flag=True, help="Do not attempt to validate registry/creds on registry add")
@click.pass_obj
def upd(cnf, registry, registry_user, registry_pass, insecure, registry_type, skip_validate):
    """
    REGISTRY: Full hostname/port of registry. Eg. myrepo.example.com:5000

    REGISTRY_USER: Username

    REGISTRY_PASS: Password
    """
    ok, res = cnf.sdscanning.update_registry(
        registry, registry_user, registry_pass,
        insecure=insecure,
        registry_type=registry_type,
        validate=not skip_validate)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res[0], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@registry.command(name='del', short_help="Delete a registry")
@click.argument('registries', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, registries):
    """
    REGISTRIES: Full hostname/port of registries. Eg. myrepo.example.com:5000
    """
    for registry in registries:
        ok, res = cnf.sdscanning.delete_registry(registry)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")


@registry.command(name='list', short_help="List all current registries")
@click.pass_obj
def list(cnf):
    ok, res = cnf.sdscanning.list_registry()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@registry.command(name='get', short_help="Get a registry")
@click.argument('registry', nargs=1, required=True)
@click.pass_obj
def get(cnf, registry):
    """
    REGISTRY: Full hostname/port of registry. Eg. myrepo.example.com:5000
    """
    ok, res = cnf.sdscanning.get_registry(registry)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res[0], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)



import json

from sdccli.printer import print_list as _print_list, print_item as _print_item


def formats():
    return {
        "dashboardList": print_dashboard_list,
        "dashboard": print_dashboard,
    }


_list_keys = ["id", "name", "username", "autoCreated", "shared", "public", "favorite"]
_item_keys = ["id", "name", "username", "autoCreated", "shared", "public", "favorite", "filterExpression"]


def print_dashboard(dashboard):
    _print_item(dashboard, _item_keys)
    if "widgets" not in dashboard:
        return

    print("Panels:")
    items = dashboard["widgets"]
    for item in items:
        if "metrics" in item:
            item["metrics"] = [m["id"] for m in item["metrics"]]
        else:
            item["metrics"] = []
    _print_list(items, ["", "name", "showAs", "metrics"])


def print_dashboard_list(response):
    _print_list(response, _list_keys)



import sys

import click

import sdccli.usecases.dashboard.panel as use_case


@click.group(name='panel', short_help='Sysdig Monitor Dashboard Panel operations')
def panel():
    pass


@panel.command(name='list', short_help="List all the panels in a dashboard")
@click.argument('dashboard', nargs=1)
@click.pass_obj
def list_panels(cnf, dashboard):
    '''
    List panels in a dashboard

    \b
    DASHBOARD   Dashboard ID to list the panels from.
    '''
    try:
        panels = use_case.get_panels_from_dashboard(cnf.sdmonitor, dashboard)
        cnf.formatter.format(panels, "panelList")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@panel.command(name='get', short_help="Get the info from a panel")
@click.argument('dashboard', nargs=1, type=click.INT)
@click.argument('panel', nargs=1, type=click.INT)
@click.pass_obj
def get(cnf, dashboard, panel):
    '''
    Retrieve information from a panel in a dashboard

    \b
    DASHBOARD   Dashboard ID where the panel lives in.
    PANEL       Panel ID within the Dashboard.
    '''
    try:
        panels = use_case.get_panels_from_dashboard(cnf.sdmonitor, dashboard)
        panel_list = [p for p in panels if p["id"] == panel]
        if len(panel_list) == 0:
            raise Exception(f"panel with id {panel} not found in dashboard {dashboard}")

        cnf.formatter.format(panel_list[0], "panel")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@panel.command(name='add', short_help="Add panel")
@click.argument('dashboard', nargs=1, type=click.INT)
@click.argument('name', nargs=1)
@click.argument('type', nargs=1)
@click.argument('query', nargs=1)
@click.pass_obj
def add(cnf, dashboard, name, type, query):
    '''
    Adds a panel to a dashboard.

    \b
    DASHBOARD   The ID of the dashboard to add a panel to.
    NAME        Name of the new panel.
    TYPE        Kind of panel, must be one of: [timechart, number].
    QUERY       PromQL query to add to the new panel.
    '''
    valid_types = {
        "timechart": use_case.PANEL_VISUALIZATION_TIMECHART,
        "number": use_case.PANEL_VISUALIZATION_NUMBER,
    }
    type = type.lower()
    if type not in valid_types:
        print(f"Type {type} is not valid, valid types are: {' '.join(valid_types.keys())}")

    try:
        panel = use_case.create_panel_in_dashboard(cnf.sdmonitor, dashboard, name, valid_types[type], query)
        cnf.formatter.format(panel, "panel")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@panel.command(name='del', short_help="Delete a panel from a dashboard")
@click.argument('dashboard', nargs=1, type=click.INT)
@click.argument('panel', nargs=1, type=click.INT)
@click.pass_obj
def delete(cnf, dashboard, panel):
    '''
    Delete a panel from a dashboard.

    \b
    DASHBOARD   Dashboard ID to remove the panel from.
    PANEL       Panel ID within the dashboard.
    '''
    try:
        use_case.delete_panel_from_dashboard(cnf.sdmonitor, dashboard, panel)
        print("Success")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)


@panel.command(name='add_query', short_help="Add query to a panel")
@click.argument('dashboard', nargs=1, type=click.INT)
@click.argument('panel', nargs=1, type=click.INT)
@click.argument('query', nargs=1)
@click.pass_obj
def add_query(cnf, dashboard, panel, query):
    '''
    Adds a query to an existing panel

    \b
    DASHBOARD   Dashboard ID where the panel is located
    PANEL       Panel ID within the dashboard
    QUERY       PromQL query to add. Check https://docs.sysdig.com/en/using-promql.html for more info.
    '''
    try:
        use_case.add_query_to_panel(cnf.sdmonitor, dashboard, panel, query)
        panel = [p for p in use_case.get_panels_from_dashboard(cnf.sdmonitor, dashboard) if p["id"] == panel][0]
        cnf.formatter.format(panel, "panel")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)

@panel.command(name='del_query', short_help="Add query to a panel")
@click.argument('dashboard', nargs=1, type=click.INT)
@click.argument('panel', nargs=1, type=click.INT)
@click.argument('query', nargs=1, type=click.INT)
@click.pass_obj
def del_query(cnf, dashboard, panel, query):
    '''
    Removes a query from an existing panel.

    \b
    DASHBOARD   Dashboard ID where the panel is located.
    PANEL       Panel ID within the dashboard.
    QUERY       Query ID within the panel. The first one is 0, the second one is 1, and so on.
                Use 'sdc-cli dashboard panel get <DASHBOARD> <PANEL>' to list all the queries in the panel
                with their IDs.
    '''
    try:
        use_case.del_query_from_panel(cnf.sdmonitor, dashboard, panel, query)
        panel = [p for p in use_case.get_panels_from_dashboard(cnf.sdmonitor, dashboard) if p["id"] == panel][0]
        cnf.formatter.format(panel, "panel")
    except Exception as ex:
        print(f"Error: {str(ex)}")
        sys.exit(1)




import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["accessKey", "enabled"]


@click.group(name='access-key', short_help='Sysdig access key operations')
def access_key():
    pass


@access_key.command(name='add', short_help="Create a new access key")
@click.pass_obj
def add(cnf):
    ok, res = cnf.sd.create_access_key()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res['customerAccessKey'], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@access_key.command(name='list', short_help="List all access keys")
@click.pass_obj
def lst(cnf):
    ok, res = cnf.sd.list_access_keys()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res['customerAccessKeys'], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@access_key.command(name='disable', short_help="Disable an access key")
@click.argument('access_key', nargs=1)
@click.pass_obj
def disable(cnf, access_key):
    """
    ACCESS_KEY: the access key to be disabled.
    """
    ok, res = cnf.sd.disable_access_key(access_key)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res['customerAccessKey'], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@access_key.command(name='enable', short_help="Enable an access key")
@click.argument('access_key', nargs=1)
@click.pass_obj
def enable(cnf, access_key):
    """
    ACCESS_KEY: the access key to be enabled.
    """
    ok, res = cnf.sd.enable_access_key(access_key)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res['customerAccessKey'], _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)



import re
import sys


class SortMacrosByDepedency(object):
    def __init__(self, macros):
        self.macros = macros

    # Sort macros by dependency from the most independent to the most dependent ones
    def sort(self, reverse=False):
        weights = SortMacrosByDepedency._gen_weighted_macro_list(self.macros)
        for macro in self.macros:
            macro['weight'] = weights[macro['name']]
        sorted_macros_by_weight = sorted(self.macros, key=lambda macro: macro['weight'], reverse=reverse)
        return sorted_macros_by_weight

    # Generates a list of key => value where the key is the name of the macro
    # and the value is a set of macros that depend on the key.
    @staticmethod
    def _gen_macro_dependency_graph(macros):
        graph = {}

        for macro in macros:
            if macro['name'] not in graph:
                graph[macro['name']] = set()

            regex = re.compile(f"[^\\w]*{macro['name']}[^\\w]*")
            for macro_to_compare in macros:
                if macro['name'] == macro_to_compare['name']:
                    continue

                # If the condition of macro_to_compare has a regex matching the macro name,
                # then macro is a dependency of macro_to_compare.
                if regex.search(macro_to_compare['condition']['condition']) is not None:
                    graph[macro['name']].add(macro_to_compare['name'])

        return graph

    # Based on the macro dependency graph, it calculates the weight of
    # each macro based on the order in which they should be removed and no
    # dependency errors should arise
    # For example those macros that are not used by any other macro, can be
    # instantly removed and will have a weight of 0. Those that depend on weight 0 will
    # have weight 1 and so on.
    # If a macro depends on multiple macros, the weight of this macro will be
    # at least higher than the max weight of its dependencies.
    @staticmethod
    def _gen_weighted_macro_list(macros):
        graph = SortMacrosByDepedency._gen_macro_dependency_graph(macros)
        weights = {}

        def calculate_macro_weight(macro_name, macro_dependencies):
            if macro_name in weights:
                return weights[macro_name]
            if len(macro_dependencies) == 0:
                weights[macro_name] = 0
                return 0
            dependencies_weight = [calculate_macro_weight(dependency, graph[dependency]) for dependency in
                                   macro_dependencies]

            weight = max(dependencies_weight) + 1
            weights[macro_name] = weight
            return weight

        for macro_name, macro_dependencies in graph.items():
            calculate_macro_weight(macro_name, macro_dependencies)

        return weights



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["id", "name", "items", "filename"]
_list_keys = ["ids", "name"]


@click.group(name='falco-list', short_help='Sysdig Secure policy falco list operations')
def falco_list():
    pass


@falco_list.command(name='add', short_help="Add a falco list")
@click.argument('name', nargs=1)
@click.option('--item', nargs=1, multiple=True, help="An item as represented in the yaml List.")
@click.pass_obj
def add(cnf, name, item):
    """
    NAME: the name of the new falco list.
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.add_falco_list(name, item if item else [])

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_list.command(name='list', short_help="List all falco lists")
@click.pass_obj
def lst(cnf):
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.list_falco_lists()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_list.command(name='get', short_help="Get falco list")
@click.argument('falco_list', nargs=1)
@click.pass_obj
def get(cnf, falco_list):
    """
    FALCO_LIST: Falco list (id or name) to get
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    try:
        id = int(falco_list)
        ok, res = cnf.sdsecure.get_falco_list_id(id)
    except ValueError:
        ok, res = cnf.sdsecure.get_falco_lists_group(falco_list)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if isinstance(res, list):
            for rule in res:
                print_item(rule, _item_keys)
        else:
            print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_list.command(name='update', short_help="Update falco list")
@click.argument('falco_list', nargs=1)
@click.option('--item', nargs=1, multiple=True, help="An item as represented in the yaml List. I twill replace the existing items in the list.")
@click.pass_obj
def upd(cnf, falco_list, item):
    """
    FALCO_LIST: Falco list id to update
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.update_falco_list(falco_list, item if item else [])

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@falco_list.command(name='del', short_help="Delete falco lists")
@click.argument('falco_lists', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, falco_lists):
    """
    FALCO_LISTS: Falco list ids to delete
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    for fl in falco_lists:
        ok, res = cnf.sdsecure.delete_falco_list(fl)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")



import click
import json
import sys
from dateutil.parser import parse
from sdccli.printer import print_list


_list_keys = ["imageId", "tag", "repo", "policyEvalStatus"]


@click.group(name='runtime', short_help='Runtime operations')
def runtime():
    pass


@runtime.command(name='list', short_help="List runtime containers")
@click.option('--scope', nargs=1,
              help=("An AND-composed string of predicates that selects the scope in which the alert will be "
                    "applied. (like: 'kubernetes.namespace.name = \"example-java-app\" and "
                    "kubernetes.deployment.name = \"example-java-app-redis\"')"))
@click.option('--skip-policy', is_flag=True, help="No policy evaluations will be triggered for the images.")
@click.option('--start', nargs=1, help="Start of the time range.")
@click.option('--end', nargs=1, help="End of the time range.")
@click.pass_obj
def list(cnf, scope, skip_policy, start, end):
    start_time = None
    if start:
        start_time = parse(start).timestamp()
    end_time = None
    if end:
        end_time = parse(end).timestamp()

    ok, res = cnf.sdscanning.list_runtime(
        scope=scope,
        skip_policy_evaluation=skip_policy,
        start_time=start_time,
        end_time=end_time)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res["images"], _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)



from multiprocessing.pool import ThreadPool

from sdcclient import SdSecureClient

from sdccli.falco_macro import SortMacrosByDepedency

ORIGIN_SECURE_UI = 'Secure UI'
ORIGIN_SYSDIG = 'Sysdig'


def dashboards(sdmonitor, remove_unowned=False):
    ok, data = sdmonitor.get_dashboards()
    if not ok:
        return False, data

    ok, userdata = sdmonitor.get_user_info()
    if not ok:
        return False, userdata

    username = userdata["user"]["username"]
    res = []
    dashboards = [dboard for dboard in data['dashboards'] if
                  remove_unowned or "username" in dboard and username in dboard["username"]]
    errors = []
    total = len(dashboards)
    for i, dashboard in enumerate(dashboards):
        print("[{}/{}] Removing dashboard {}... ".format(
            i + 1, total, dashboard['name'].strip()))
        removed, err = sdmonitor.delete_dashboard(dashboard)
        if removed:
            res.append(dashboard)
        else:
            errors.append(err)

    if errors:
        return False, errors

    return True, res


def alerts(sdmonitor):
    ok, data = sdmonitor.get_alerts()
    if not ok:
        return False, data

    res = []
    errors = []
    total = len(data['alerts'])
    for i, alert in enumerate(data['alerts'], start=1):
        print(f"[{i}/{total}] Removing alert {alert['name'].strip()}...")
        removed, err = sdmonitor.delete_alert(alert)
        if removed:
            res.append(alert)
        else:
            errors.append(err)

    if len(errors) > 0:
        return False, errors
    return True, res


# TODO: Move this to the python-sdc-client
def get_user_created_rules(secure: SdSecureClient):
    ok, res = secure.list_rules()
    if not ok:
        return False, res

    user_created_rules = []
    for list in res:
        for j, id in enumerate(list['ids']):
            if list['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                user_rules = list.copy()

                user_rules['id'] = list['ids'][j]
                del user_rules['ids']

                user_rules['publishedBy'] = list['publishedBys'][j]
                del user_rules['publishedBys']

                user_created_rules.append(user_rules)

    return True, user_created_rules


def rules(sdsecure):
    ok, data = get_user_created_rules(sdsecure)
    if not ok:
        return False, data

    result = []
    errors = []
    total = len(data)
    for i, rule in enumerate(data, start=1):
        print(f"[{i}/{total}] Removing rule {rule['name'].strip()}...")
        ok, res = sdsecure.delete_rule(rule['id'])
        if not ok:
            errors.append(res)
            continue
        result.append(rule)

    if len(errors) > 0:
        return False, errors
    return True, result


# TODO: Move this to the python-sdc-client
def get_user_created_lists(secure: SdSecureClient):
    ok, res = secure.list_falco_lists()
    if not ok:
        return False, res

    user_created_lists = []
    for list in res:
        for j, id in enumerate(list['ids']):
            if list['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                user_list = list.copy()

                user_list['id'] = list['ids'][j]
                del user_list['ids']

                user_list['publishedBy'] = list['publishedBys'][j]
                del user_list['publishedBys']

                user_created_lists.append(user_list)

    return True, user_created_lists


def falco_lists(sdsecure: SdSecureClient):
    ok, data = get_user_created_lists(sdsecure)
    if not ok:
        return False, data

    errors = []
    result = []
    total = len(data)
    for i, flist in enumerate(data, start=1):
        print(f"[{i}/{total}] Removing falco list {flist['name'].strip()}...")
        ok, res = sdsecure.delete_falco_list(flist['id'])
        if not ok:
            errors.append(res)
            continue
        result.append(flist)

    if len(errors) > 0:
        return False, errors
    return True, result


# TODO: Move this method to the python-sdc-client
# This only creates a single API call
def get_user_created_macros(secure: SdSecureClient):
    ok, res = secure.list_falco_macros()
    if not ok:
        return False, res

    user_created_macros = []
    for macro in res:
        for j, id in enumerate(macro['ids']):
            if macro['publishedBys'][j]['origin'] == ORIGIN_SECURE_UI:
                user_macro = macro.copy()

                user_macro['id'] = macro['ids'][j]
                del user_macro['ids']

                user_macro['publishedBy'] = macro['publishedBys'][j]
                del user_macro['publishedBys']

                user_created_macros.append(user_macro)

    return True, user_created_macros


# TODO: Move this method to the python-sdc-client
# This creates N API calls where N is the number of user created macros.
# This is because the information retrieved by list_falco_macros() is incomplete.
# If one wants to retrieve the conditions of every macro and not only the names, they need to
# retrieve one by one.
def get_full_user_created_macros(secure: SdSecureClient):
    ok, macros = get_user_created_macros(secure)
    # usedCount field is always 0 no matter if this macro is being used in a policy or by other macro :/
    if not ok:
        return False, macros

    errors = []
    full_macros = []
    for ok, res in ThreadPool().map(secure.get_falco_macro_id, (macro['id'] for macro in macros)):
        if not ok:
            errors.append(res)
        else:
            full_macros.append(res)

    if len(errors) > 0:
        return False, errors
    return True, full_macros


def falco_macros(sdsecure: SdSecureClient):
    ok, macros = get_full_user_created_macros(sdsecure)
    if not ok:
        return False, macros

    sorted_macros_by_dependency = SortMacrosByDepedency(macros).sort()

    errors = []
    total = len(sorted_macros_by_dependency)
    for i, macro in enumerate(sorted_macros_by_dependency, start=1):
        print(f"[{i}/{total}] Removing falco macro {macro['name'].strip()}...")
        ok, res = sdsecure.delete_falco_macro(macro['id'])
        if not ok:
            errors.append(res)

    if len(errors) > 0:
        return False, errors

    return True, sorted_macros_by_dependency


def policies(sdsecure: SdSecureClient):
    ok, data = sdsecure.list_policies()
    if not ok:
        return False, data
    # Policies are the only resource that can be removed with an origin different from "Secure UI".
    # Policies from "Sysdig" can also be removed. This does not happen with rules, lists, etc.
    removable_policies = [policy for policy in data if policy['origin'] in [ORIGIN_SECURE_UI, ORIGIN_SYSDIG]]

    errors = []
    res = []
    total = len(removable_policies)

    for i, policy in enumerate(removable_policies, start=1):
        print(f"[{i}/{total}] Removing policy {policy['name'].strip()}... ")
        removed, err = sdsecure.delete_policy_id(policy['id'])
        if removed:
            res.append(policy)
        else:
            errors.append(err)

    if len(errors) > 0:
        return False, errors
    return True, res


def teams(sdmonitor):
    ok, data = sdmonitor.get_teams()
    if not ok:
        return False, data

    res = []
    errors = []
    mutable_teams = [team for team in data if not team['immutable'] and not team['default']]
    total = len(mutable_teams)
    for i, team in enumerate(mutable_teams, start=1):
        print(f"[{i}/{total}] Removing team {team['name'].strip()}... ")
        removed, err = sdmonitor.delete_team(team['name'])
        if removed:
            res.append(team)
        else:
            errors.append(res)

    if len(errors) > 0:
        return False, errors
    return True, res


def notification_channels(sdmonitor):
    ok, data = sdmonitor.list_notification_channels()
    if not ok:
        return False, data

    res = []
    total = len(data['notificationChannels'])
    for i, channel in enumerate(data['notificationChannels']):
        print("[{}/{}] Removing notification channel {}... ".format(
            i + 1, total, channel['name']))
        removed, err = sdmonitor.delete_notification_channel(channel)
        if removed:
            res.append(channel)
        else:
            print("Error: " + err)

    return True, res



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sdccli',
 'sdccli.cli',
 'sdccli.cli.dashboard',
 'sdccli.cli.formatter',
 'sdccli.cli.formatter.json_formatter',
 'sdccli.cli.formatter.text_formatter',
 'sdccli.cli.formatter.text_formatter.scanning',
 'sdccli.cli.policy',
 'sdccli.cli.scanning',
 'sdccli.cli.scanning.vulnerability',
 'sdccli.cli.settings',
 'sdccli.usecases',
 'sdccli.usecases.backup',
 'sdccli.usecases.dashboard',
 'sdccli.usecases.policy',
 'sdccli.usecases.scanning',
 'sdccli.usecases.settings']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0',
 'prettytable>=2,<3',
 'python-dateutil>=2.8.1,<3.0.0',
 'pyyaml>=5.3.1,<6.0.0',
 'requests>=2,<3',
 'sdcclient>=0.16.1,<0.17.0']

entry_points = \
{'console_scripts': ['sdc-cli = sdccli.cli:cli']}

setup_kwargs = {
    'name': 'sdccli',
    'version': '0.7.14',
    'description': 'CLI client for Sysdig Cloud',
    'long_description': None,
    'author': 'Sysdig Inc.',
    'author_email': 'info@sysdig.com',
    'maintainer': 'Nestor Salceda',
    'maintainer_email': 'nestor.salceda@sysdig.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import click
import json
import sys
from prettytable import PrettyTable, PLAIN_COLUMNS
from sdccli.time import date_range_in_sec, format_timestamp


@click.group(name='command', short_help="Sysdig Secure commands audit operations")
def command():
    pass


@command.command(name='list', short_help="List all commands")
@click.option('--duration', nargs=1, help="Duration to display the commands from. (ex: 30M, 1H, 3D, 2W)")
@click.option('--start', nargs=1, help="Start of the time range.")
@click.option('--end', nargs=1, help="End of the time range.")
@click.option('--scope', nargs=1, help="this is a SysdigMonitor-like filter (e.g 'container.image=ubuntu'). When provided, events are filtered by their scope, so only a subset will be returned (e.g. 'container.image=ubuntu' will provide only events that have happened on an ubuntu container).")
@click.option('--filter', nargs=1, help='this is a SysdigMonitor-like filter (e.g. command.comm="touch"). When provided, commands are filtered by some of their properties. Currently the supported set of filters is command.comm, command.cwd, command.pid, command.ppid, command.uid, command.loginshell.id, command.loginshell.distance')
@click.option('--limit', nargs=1, default=100, help='Maximum number of commands to list. If is not provided it will list up to 100 commands.')
@click.pass_obj
def list(cnf, duration, start, end, scope, filter, limit):
    try:
        from_sec, to_sec = date_range_in_sec(start, end, duration)
    except IndexError as ex:
        print("Error: {}".format(ex))
        sys.exit(1)

    ok, res = cnf.sdsecure.list_commands_audit(
        from_sec=from_sec,
        to_sec=to_sec,
        scope_filter=scope,
        command_filter=filter,
        metrics=["host.hostName"],
        limit=limit)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_commands(res['commands'])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@command.command(name='get', short_help="Get a command audit")
@click.argument('command-id', nargs=1)
@click.pass_obj
def get(cnf, command_id):
    """
    COMMAND-ID: the id of the command audit to get.
    """
    ok, res = cnf.sdsecure.get_command_audit(
        command_id,
        metrics=["host.hostName", "host.mac", "container.image"])

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_command(res['command'])
    else:
        print("Error: " + str(res))
        sys.exit(1)


def _print_command(command):
    print("{:25} {}".format("id:", command["id"]))
    print("{:25} {}".format("time:", format_timestamp(command["timestamp"])))
    print("{:25} {}".format("command:", command["cmdline"]))
    print("{:25} {}".format("working directory:", command["cwd"]))
    host = "%s (%s)" % (command["metrics"][0], command["metrics"][1])
    print("{:25} {}".format("host:", host))
    if "containerId" in command:
        container = "%s (%s)" % (command["metrics"][2], command["containerId"])
        print("{:25} {}".format("container:", container))
    print("{:25} {}".format("pid:", command["pid"]))
    print("{:25} {}".format("ppid:", command["ppid"]))
    print("{:25} {}".format("user id:", command["uid"]))
    print("{:25} {}".format("shell id:", command["loginShellId"]))
    print("{:25} {}".format("shell distance:", command["loginShellDistance"]))


def _print_commands(commands):
    t = PrettyTable(["id", "time", "command", "host"])
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for command in commands:
        values = []
        values.append(command["id"])
        values.append(format_timestamp(command["timestamp"]))
        values.append(command["comm"])
        host = command["metrics"][0]
        if "containerId" in command:
            host += " > " + command["containerId"]
        values.append(host)
        t.add_row(values)
    print(t.get_string())



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["id", "name", "description", "default", "show", "filter", "canUseSysdigCapture", "canUseCustomEvents", "canUseAwsMetrics"]
_list_keys = ["id", "name", "default", "userCount"]


@click.group(name='team', short_help='Sysdig team operations')
def team():
    pass


@team.command(name='add', short_help="Create a new team")
@click.argument('name', nargs=1)
@click.option('--member', nargs=1, multiple=True, help="a user name or pair user-name=team-role of a member of the team")
@click.option('--filter', nargs=1, default='', help="the scope that this team is able to access within Sysdig Monitor.")
@click.option('--description', nargs=1, default='', help="describes the team that will be created.")
@click.option('--show', nargs=1, default='host', help="possible values are *host*, *container*.")
@click.option('--theme', nargs=1, default='#7BB0B2', help="the color theme that Sysdig will use when displaying the team. In #RRGGBB form.")
@click.option('--perm-capture', is_flag=True, help="if True, this team will be allowed to take sysdig captures.")
@click.option('--perm-custom-events', is_flag=True, help="if True, this team will be allowed to view all custom events from every user and agent.")
@click.option('--perm-aws-data', is_flag=True, help="if True, this team will have access to all AWS metrics and tags, regardless of the team's scope.")
@click.pass_obj
def add(cnf, name, member, filter, description, show, theme, perm_capture, perm_custom_events, perm_aws_data):
    """
    NAME: the name of the team to create.
    """
    memberships = None
    if member:
        memberships = {}
        for m in member:
            if '=' in m:
                (k, v) = m.split('=', 1)
                memberships[k] = v
            else:
                memberships[m] = "ROLE_TEAM_EDIT"

    ok, res = cnf.sd.create_team(
        name,
        memberships=memberships,
        filter=filter,
        description=description,
        show=show,
        theme=theme,
        perm_capture=perm_capture,
        perm_custom_events=perm_custom_events,
        perm_aws_data=perm_custom_events)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_team(res["team"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@team.command(name='del-member', short_help="Remove a member from a team")
@click.argument('team', nargs=1)
@click.argument('user-name', nargs=1)
@click.pass_obj
def del_member(cnf, team, user_name):
    """
    TEAM: the name of the team to remove the member from.
    USER-NAME: the name of the user to remove.
    """
    ok, res = cnf.sd.remove_memberships(team, [user_name])

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print("Success")
    else:
        print("Error: " + str(res))
        sys.exit(1)


@team.command(name='add-member', short_help="Add a member to a team")
@click.argument('team', nargs=1)
@click.argument('user-name', nargs=1)
@click.option('--role', nargs=1, default='ROLE_TEAM_EDIT', help="The role for the user in the team. (default 'ROLE_TEAM_EDIT')")
@click.pass_obj
def add_member(cnf, team, user_name, role):
    """
    TEAM: the name of the team to add the member to.
    USER-NAME: the name of the user to add.
    """
    ok, res = cnf.sd.save_memberships(team, memberships={user_name: role})

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print("Success")
    else:
        print("Error: " + str(res))
        sys.exit(1)


@team.command(name='list', short_help="List all teams")
@click.pass_obj
def list(cnf):
    ok, res = cnf.sd.get_teams()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@team.command(name='update', short_help="Update the team")
@click.argument('name', nargs=1)
@click.option('--member', nargs=1, multiple=True, help="a user name or pair user-name=team-role of a member of the team")
@click.option('--filter', nargs=1, default=None, help="the scope that this team is able to access within Sysdig Monitor.")
@click.option('--description', nargs=1, default='', help="describes the team that will be created.")
@click.option('--show', nargs=1, default='host', help="possible values are *host*, *container*.")
@click.option('--theme', nargs=1, default='#7BB0B2', help="the color theme that Sysdig will use when displaying the team. In #RRGGBB form.")
@click.option('--perm-capture', is_flag=True, help="if True, this team will be allowed to take sysdig captures.")
@click.option('--perm-custom-events', is_flag=True, help="if True, this team will be allowed to view all custom events from every user and agent.")
@click.option('--perm-aws-data', is_flag=True, help="if True, this team will have access to all AWS metrics and tags, regardless of the team's scope.")
@click.pass_obj
def upd(cnf, name, member, filter, description, show, theme, perm_capture, perm_custom_events, perm_aws_data):
    """
    NAME: the name of the team to update.
    """
    memberships = None
    if member:
        memberships = {}
        for m in member:
            if '=' in m:
                (k, v) = m.split('=', 1)
                memberships[k] = v
            else:
                memberships[m] = "ROLE_TEAM_EDIT"

    ok, res = cnf.sd.edit_team(
        name,
        memberships=memberships,
        filter=filter,
        description=description,
        show=show,
        theme=theme,
        perm_capture=perm_capture,
        perm_custom_events=perm_custom_events,
        perm_aws_data=perm_custom_events)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_team(res["team"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@team.command(name='get', short_help="Get team")
@click.argument('team', nargs=1)
@click.pass_obj
def get(cnf, team):
    """
    TEAM: the team name or id to get
    """
    try:
        id = int(team)
        ok, res = cnf.sd.get_teams()
        if ok:
            found = False
            for t in res:
                if id == t['id']:
                    res = t
                    found = True
                    break
            if not found:
                print("Error: no team with id {}".format(id))
                sys.exit(1)
                return
    except ValueError:
        ok, res = cnf.sd.get_team(team)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_team(res)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@team.command(name='del', short_help="Delete users")
@click.argument('names', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, names):
    """
    NAMES: The names of the teams to delete
    """
    for name in names:
        ok, res = cnf.sd.delete_team(name)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")


def _print_team(team):
    print_item(team, _item_keys)
    print("{:25} {}".format("Entry point:", team["entryPoint"]["module"]))
    print("")
    print_list(team["userRoles"], ["userId", "userName", "role", "admin"])



import json
import sys
from datetime import datetime

import click
from prettytable import PrettyTable, PLAIN_COLUMNS

from sdccli.printer import print_item
from sdccli.time import format_timestamp, duration_to_timedelta


@click.group(name='event', short_help="Sysdig Monitor events operations")
def event():
    pass


@event.command(name='add', short_help="Add an event")
@click.argument('name', nargs=1)
@click.option('--description', nargs=1, help="a longer description offering detailed information about the event.")
@click.option('--severity', nargs=1, type=click.INT, help="syslog style from 0 (high) to 3 (low).")
@click.option('--filter', nargs=1,
              help="metadata, in Sysdig Monitor format, of nodes to associate with the event, e.g. ``host.hostName = 'ip-10-1-1-1' and container.name = 'foo'``.")
@click.option('--tag', nargs=1,
              help="A key=value that can be used to tag the event. Can be used for filtering/segmenting purposes in Sysdig Monitor.")
@click.pass_obj
def add(cnf, name, description, severity, filter, tag):
    """
    NAME: the name of the new event.
    """
    tags = None
    if tag:
        tags = {}
        for t in tag:
            (k, v) = t.split('=', 1) if '=' in t else ('', '')
            if not k or not v:
                print("tag format error - tags must be of the form (--tag key=value), found: {}".format(t))
                return
            tags[k] = v

    if severity is not None and not -1 < severity < 4:
        print("The severity values must be between 0 (high) and 3 (low), both included.")
        sys.exit(1)

    ok, res = cnf.sdmonitor.post_event(
        name,
        description=description,
        severity=severity,
        event_filter=filter,
        tags=tags)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_event(res["event"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@event.command(name='list', short_help="List all events")
@click.option('--duration', nargs=1, help="Duration to display the events from. (ex: 30M, 1H, 3D, 2W)")
@click.option('--limit', nargs=1, type=int, help="Max number of events to print. Default: 100")
@click.option('--name', nargs=1, help="Filter events by name.")
@click.pass_obj
def list_events(cnf, duration, limit, name):
    from_sec = to_sec = None
    if duration is not None:
        delta = duration_to_timedelta(duration)
        to_sec = datetime.now()
        from_sec = to_sec - delta

    if limit is None:
        limit = 100

    ok, res = cnf.sdmonitor.get_events(
        name=name,
        from_s=from_sec,
        to_s=to_sec,
        limit=limit)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_events(res["events"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@event.command(name='get', short_help="Get an event")
@click.argument('event-id', nargs=1)
@click.pass_obj
def get(cnf, event_id):
    """
    EVENT-ID: the id of the event to display
    """
    ok, res = cnf.sdmonitor.get_event(event_id)
    if not ok:
        print("Error: " + str(res))
        sys.exit(1)

    if cnf.json:
        print(json.dumps(res["event"], indent=4))
        return

    _print_event(res["event"])


@event.command(name='del', short_help="Delete events")
@click.argument('events', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, events):
    """
    EVENTS: Event ids to delete
    """
    for event in events:
        ok, res = cnf.sdmonitor.delete_event({"id": event})

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")


def _print_events(events):
    t = PrettyTable(["id", "severity", "name", "date", "source"])
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for event in events:
        values = []
        values.append(event["id"])
        values.append(event["severity"])
        values.append(event["name"])
        values.append(format_timestamp(event["timestamp"]))
        if "source" in event:
            values.append(event["source"])
        elif "type" in event and event["type"]:
            values.append(event["type"].lower())
        else:
            values.append("NONE")
        t.add_row(values)
    print(t.get_string())


def _print_event(event):
    print_item(event, ["name", "id", "severityLabel", "description", "filter"])
    print("{:25} {}".format("date:", format_timestamp(event["timestamp"])))
    if "tags" in event and "source" in event["tags"]:
        print("{:25} {}".format("source:", event["tags"]["source"]))



import click
import json
import sys
from sdccli.printer import print_list, print_item


_item_keys = ["id", "name", "description", "details", "tags", "filename"]
_list_keys = ["ids", "name", "ruleType"]


@click.group(name='rule', short_help='Sysdig Secure policy rule operations')
def rule():
    pass


@rule.command(name='add-container', short_help="Add a container rule")
@click.argument('name', nargs=1)
@click.argument('description', nargs=1)
@click.option('--not-matching', is_flag=True, help="The rule matches if the container does not match any of the items in the list.")
@click.option('--container', nargs=1, multiple=True)
@click.option('--tag', nargs=1, multiple=True, help="A rule tag.")
@click.pass_obj
def add_container(cnf, name, description, not_matching, container, tag):
    """
    NAME: the name of the new rule.

    DESCRIPTION: Description of rule
    """
    details = {
        'containers': {
            'items': container if container else [],
            'matchItems': not not_matching
        },
        'ruleType': 'CONTAINER'
    }
    add(cnf, name, description, details, tag)


@rule.command(name='add-file', short_help="Add a file system rule")
@click.argument('name', nargs=1)
@click.argument('description', nargs=1)
@click.option('--rw-not-matching', is_flag=True, help="The rule matches if the read/write path does not match any of the items in the list.")
@click.option('--rw-path', nargs=1, multiple=True, help="read/write path")
@click.option('--ro-not-matching', is_flag=True, help="The rule matches if the read only path does not match any of the items in the list.")
@click.option('--ro-path', nargs=1, multiple=True, help="read only path")
@click.option('--tag', nargs=1, multiple=True, help="A rule tag.")
@click.pass_obj
def add_file(cnf, name, description, rw_not_matching, rw_path, ro_not_matching, ro_path, tag):
    """
    NAME: the name of the new rule.

    DESCRIPTION: Description of rule
    """
    details = {
        'readPaths': {
            'items': ro_path if ro_path else [],
            'matchItems': not ro_not_matching
        },
        'readWritePaths': {
            'items': rw_path if rw_path else [],
            'matchItems': not rw_not_matching
        },
        'ruleType': 'FILESYSTEM'
    }
    add(cnf, name, description, details, tag)


@rule.command(name='add-network', short_help="Add a network rule")
@click.argument('name', nargs=1)
@click.argument('description', nargs=1)
@click.option('--inbound-deny', is_flag=True, help="If set all inbound connection attempts match this rule")
@click.option('--outbound-deny', is_flag=True, help="If set all outbound connection attempts match this rule")
@click.option('--tcp-not-matching', is_flag=True, help="The rule matches if the tcp port does not match any of the items in the list.")
@click.option('--tcp-port', nargs=1, multiple=True, help="tcp port")
@click.option('--udp-not-matching', is_flag=True, help="The rule matches if the udp port does not match any of the items in the list.")
@click.option('--udp-port', nargs=1, multiple=True, help="udp port")
@click.option('--tag', nargs=1, multiple=True, help="A rule tag.")
@click.pass_obj
def add_network(cnf, name, description, inbound_deny, outbound_deny, tcp_not_matching, tcp_port, udp_not_matching, udp_port, tag):
    """
    NAME: the name of the new rule.

    DESCRIPTION: Description of rule
    """
    details = {
        'allInbound': inbound_deny,
        'allOutbound': outbound_deny,
        'tcpListenPorts': {
            'items': tcp_port if tcp_port else [],
            'matchItems': not tcp_not_matching
        },
        'udpListenPorts': {
            'items': udp_port if udp_port else [],
            'matchItems': not udp_not_matching
        },
        'ruleType': 'NETWORK'
    }
    add(cnf, name, description, details, tag)


@rule.command(name='add-process', short_help="Add a process rule")
@click.argument('name', nargs=1)
@click.argument('description', nargs=1)
@click.option('--not-matching', is_flag=True, help="The rule matches if the process does not match any of the items in the list.")
@click.option('--process', nargs=1, multiple=True)
@click.option('--tag', nargs=1, multiple=True, help="A rule tag.")
@click.pass_obj
def add_process(cnf, name, description, not_matching, process, tag):
    """
    NAME: the name of the new rule.

    DESCRIPTION: Description of rule
    """
    details = {
        'processes': {
            'items': process if process else [],
            'matchItems': not not_matching
        },
        'ruleType': 'PROCESS'
    }
    add(cnf, name, description, details, tag)


@rule.command(name='add-syscall', short_help="Add a syscall rule")
@click.argument('name', nargs=1)
@click.argument('description', nargs=1)
@click.option('--not-matching', is_flag=True, help="The rule matches if the syscall does not match any of the items in the list.")
@click.option('--syscall', nargs=1, multiple=True)
@click.option('--tag', nargs=1, multiple=True, help="A rule tag.")
@click.pass_obj
def add_syscall(cnf, name, description, not_matching, syscall, tag):
    """
    NAME: the name of the new rule.

    DESCRIPTION: Description of rule
    """
    details = {
        'syscalls': {
            'items': syscall if syscall else [],
            'matchItems': not not_matching
        },
        'ruleType': 'SYSCALL'
    }
    add(cnf, name, description, details, tag)


@rule.command(name='add-falco', short_help="Add a falco rule")
@click.argument('name', nargs=1)
@click.argument('description', nargs=1)
@click.option('--condition', nargs=1, help="the full condition text exactly as represented in the yaml file")
@click.option('--output', nargs=1, help="A string describing the output string to generate when this rule matches an event. Should exactly match the output property of the rule's output field")
@click.option('--priority', nargs=1, default="warning", help="The falco rule's priority. This is only included so the resulting rule can be converted back to yaml easily. For the purposes of policy events, the policy's severity should be used instead of this value. [ emergency, alert, critical, error, warning, notice, informational, debug ]")
@click.option('--source', nargs=1, default="syscall", help="The kinds of events this rule should run on. Note that this is different that the file containing the rule. This is naming an event source. [ k8s_audit, syscall ]")
@click.option('--tag', nargs=1, multiple=True, help="A rule tag.")
@click.pass_obj
def add_falco(cnf, name, description, condition, output, priority, source, tag):
    """
    NAME: the name of the new rule.

    DESCRIPTION: Description of rule
    """
    details = {
        'append': False,
        'condition': {
            'components': [],
            'condition': condition
        },
        'output': output,
        'priority': priority,
        'source': source,
        'ruleType': 'FALCO'
    }
    add(cnf, name, description, details, tag)


def add(cnf, name, description, details, tags):
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.add_rule(
        name,
        details=details,
        description=description,
        tags=tags if tags else [])

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@rule.command(name='list', short_help="List all rules")
@click.pass_obj
def lst(cnf):
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    ok, res = cnf.sdsecure.list_rules()

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_list(res, _list_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@rule.command(name='get', short_help="Get rule")
@click.argument('rule', nargs=1)
@click.pass_obj
def get(cnf, rule):
    """
    RULE: Rule (id or name) to get
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    try:
        id = int(rule)
        ok, res = cnf.sdsecure.get_rule_id(id)
    except ValueError:
        ok, res = cnf.sdsecure.get_rules_group(rule)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        if isinstance(res, list):
            for rule in res:
                print_item(rule, _item_keys)
        else:
            print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@rule.command(name='update', short_help="Update rule")
@click.argument('rule', nargs=1)
@click.option('--description', nargs=1, default="", help="Description of rule")
@click.option('--details', nargs=1, help="The rule descrption in a json format")
@click.option('--tag', nargs=1, multiple=True, help="A rule tag. It will replace the tags defined in the rule.")
@click.pass_obj
def upd(cnf, rule, description, details, tag):
    """
    RULES: Rule id to update
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    detail_dict = json.loads(details)
    ok, res = cnf.secure.update_rule(
        rule,
        details=detail_dict,
        description=description,
        tags=tag if tag else [])

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        print_item(res, _item_keys)
    else:
        print("Error: " + str(res))
        sys.exit(1)


@rule.command(name='del', short_help="Delete rules")
@click.argument('rules', nargs=-1, required=True)
@click.pass_obj
def delete(cnf, rules):
    """
    RULES: Rule id to delete
    """
    if not cnf.sdsecure.policy_v2:
        print("Error: Sysdig Secure doesn't support policy API v2")
        sys.exit(1)

    for rule in rules:
        ok, res = cnf.sdsecure.delete_rule(rule)

        if not ok:
            print("Error: " + str(res))
            sys.exit(1)

    print("Success")



import click
import json
import sys
from datetime import datetime
from sdccli.time import date_range_in_sec
from prettytable import PrettyTable, PLAIN_COLUMNS


@click.group(name='capture', short_help='Sysdig capture operations')
@click.option('--secure', is_flag=True, help='Work Sysdig Secure captures instead of Sysdig Monitor')
@click.pass_obj
def capture(cnf, secure):
    cnf.sd = cnf.sdsecure if secure else cnf.sdmonitor


@capture.command(name='add', short_help="Create a new sysdig capture. The capture will be immediately started.")
@click.argument('name', nargs=1)
@click.argument('hostname', nargs=1)
@click.option('--duration', nargs=1, type=click.INT, default=30, help="the duration of the capture, in seconds.")
@click.option('--filter', nargs=1, default="", help="a sysdig filter expression.")
@click.option('--folder', nargs=1, default="/", help="directory in the S3 bucket where the capture will be saved.")
@click.pass_obj
def add(cnf, name, hostname, duration, filter, folder):
    """
    NAME: the name of the capture.
    HOSTNAME: the hostname of the instrumented host where the capture will be taken.
    """
    ok, res = cnf.sd.create_sysdig_capture(hostname, name, duration, filter, folder)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_capture(res["dump"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@capture.command(name='list', short_help="List alerts")
@click.option('--duration', nargs=1, help="Duration to display the events from. (ex: 30M, 1H, 3D, 2W)")
@click.option('--start', nargs=1, help="Start of the time range.")
@click.option('--end', nargs=1, help="End of the time range.")
@click.option('--scope', nargs=1, help="this is a SysdigMonitor-like filter (e.g 'container.image=ubuntu'). When provided, events are filtered by their scope, so only a subset will be returned (e.g. 'container.image=ubuntu' will provide only events that have happened on an ubuntu container).")
@click.pass_obj
def list(cnf, duration, start, end, scope):
    try:
        from_sec, to_sec = date_range_in_sec(start, end, duration)
    except IndexError as ex:
        print("Error: {}".format(ex))
        sys.exit(1)

    ok, res = cnf.sd.get_sysdig_captures(
        from_sec=from_sec,
        to_sec=to_sec,
        scope_filter=scope)

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_captures(res["dumps"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@capture.command(name='get', short_help="Fetch the state of a sysdig capture")
@click.argument('capture-id', nargs=1)
@click.pass_obj
def get(cnf, capture_id):
    """
    CAPTURE-ID: the capture id
    """
    ok, res = cnf.sd.poll_sysdig_capture({'id': capture_id})

    if cnf.json:
        print(json.dumps(res, indent=4))
        return

    if ok:
        _print_capture(res["dump"])
    else:
        print("Error: " + str(res))
        sys.exit(1)


@capture.command(name='download', short_help="Download a sysdig capture")
@click.argument('capture-id', nargs=1)
@click.argument('file-scap', nargs=1, type=click.File('wb'))
@click.pass_obj
def download(cnf, capture_id, file_scap):
    """
    CAPTURE-ID: the capture id
    FILE-SCAP: the path of the file to save the capture to
    """
    ok, res = cnf.sd.download_sysdig_capture(capture_id)
    if ok:
        file_scap.write(res)
    else:
        print("Error: " + str(res))
        sys.exit(1)


def _print_capture(capture):
    print("{:25} {}".format("id:", capture["id"]))
    print("{:25} {}".format("name:", capture["name"][:-5]))
    if "error" in capture:
        print("{:25} {}".format("error:", capture["error"]))
    print("{:25} {}".format("hostname:", capture["agent"]["hostName"]))
    if "containerId" in capture:
        print("{:25} {}".format("container id:", capture["containerId"]))
    print("{:25} {}".format("time:", _format_time(capture)))
    print("{:25} {}".format("folder:", capture["folder"]))
    if capture["size"]:
        print("{:25} {}".format("size:", _size2str(capture["size"])))
    if capture["notificationInfo"]:
        print("{:25} {}".format("notification:", capture["notificationInfo"]))
    print("{:25} {}".format("status:", capture["status"]))
    if "policyEvents" in capture:
        event_ids = [e["policyId"] for e in capture["policyEvents"]]
        print("{:25} {}".format("policy ids:", event_ids))


def _print_captures(captures):
    t = PrettyTable(["id", "name", "hostname", "time", "status"])
    t.set_style(PLAIN_COLUMNS)
    t.align = 'l'

    for capture in captures:
        values = []
        values.append(capture["id"])
        values.append(capture["name"][:-5])
        values.append(capture["agent"]["hostName"])
        values.append(_format_time(capture))
        values.append(capture["status"])
        t.add_row(values)
    print(t.get_string())


def _format_time(capture):
    date = str(_to_datetime(capture["dateRequested"]))
    if capture["dateReceived"]:
        duration = _to_seconds(capture["dateReceived"] - capture["dateRequested"])
        date += " ({:.0f} sec)".format(duration)
    return date


def _to_seconds(date):
    return date / 10**3


def _to_datetime(date):
    return datetime.utcfromtimestamp(_to_seconds(date))


def _size2str(size):
    for metric in ["B", "KiB", "MiB", "GiB"]:
        nsize = size / 1024
        if nsize < 1:
            return "{:.2f} {}".format(size, metric)
        size = nsize
    return "{:.2f} {}".format(size, metric)
